@@ -12,7 +12,9 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library. If not, see <http://www.gnu.org/licenses/>.
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
 /*
@@ -30,11 +32,9 @@
 
 #include <math.h>
 
-#include "fallback-c89.c"
-
 /* LIMITATIONS:
  *
- * Offscreen windows can’t be the child of a foreign window,
+ * Offscreen windows can't be the child of a foreign window,
  *   nor contain foreign windows
  * GDK_POINTER_MOTION_HINT_MASK isn't effective
  */
@@ -143,17 +143,18 @@ _gdk_offscreen_window_create_surface (GdkWindow *offscreen,
                                       gint       width,
                                       gint       height)
 {
-  GdkOffscreenWindow *impl;
-  GdkWindow *derived;
+  cairo_surface_t *similar;
+  cairo_surface_t *surface;
 
   g_return_val_if_fail (GDK_IS_OFFSCREEN_WINDOW (offscreen->impl), NULL);
 
-  impl = GDK_OFFSCREEN_WINDOW (offscreen->impl);
-  derived = impl->embedder ? impl->embedder : offscreen->parent;
+  similar = _gdk_window_ref_cairo_surface (offscreen->parent);
 
-  return gdk_window_create_similar_surface (derived,
-					    CAIRO_CONTENT_COLOR_ALPHA,
-					    width, height);
+  surface = cairo_surface_create_similar (similar, CAIRO_CONTENT_COLOR_ALPHA, width, height);
+
+  cairo_surface_destroy (similar);
+
+  return surface;
 }
 
 void
@@ -201,7 +202,7 @@ gdk_offscreen_window_reparent (GdkWindow *window,
   gdk_window_hide (window);
 
   if (window->parent)
-    window->parent->children = g_list_remove_link (window->parent->children, &window->children_list_node);
+    window->parent->children = g_list_remove (window->parent->children, window);
 
   old_parent = window->parent;
   window->parent = new_parent;
@@ -209,7 +210,7 @@ gdk_offscreen_window_reparent (GdkWindow *window,
   window->y = y;
 
   if (new_parent)
-    window->parent->children = g_list_concat (&window->children_list_node, window->parent->children);
+    window->parent->children = g_list_prepend (window->parent->children, window);
 
   _gdk_synthesize_crossing_events_for_geometry_change (window);
   if (old_parent)
@@ -218,13 +219,6 @@ gdk_offscreen_window_reparent (GdkWindow *window,
   return was_mapped;
 }
 
-static void
-gdk_offscreen_window_set_device_cursor (GdkWindow     *window,
-					GdkDevice     *device,
-					GdkCursor     *cursor)
-{
-}
-
 static void
 from_embedder (GdkWindow *window,
 	       double embedder_x, double embedder_y,
@@ -249,7 +243,7 @@ to_embedder (GdkWindow *window,
 			 NULL);
 }
 
-static void
+static gint
 gdk_offscreen_window_get_root_coords (GdkWindow *window,
 				      gint       x,
 				      gint       y,
@@ -281,17 +275,19 @@ gdk_offscreen_window_get_root_coords (GdkWindow *window,
     *root_x = tmpx;
   if (root_y)
     *root_y = tmpy;
+
+  return TRUE;
 }
 
 static gboolean
 gdk_offscreen_window_get_device_state (GdkWindow       *window,
                                        GdkDevice       *device,
-                                       gdouble         *x,
-                                       gdouble         *y,
+                                       gint            *x,
+                                       gint            *y,
                                        GdkModifierType *mask)
 {
   GdkOffscreenWindow *offscreen;
-  double tmpx, tmpy;
+  int tmpx, tmpy;
   double dtmpx, dtmpy;
   GdkModifierType tmpmask;
 
@@ -302,18 +298,18 @@ gdk_offscreen_window_get_device_state (GdkWindow       *window,
   offscreen = GDK_OFFSCREEN_WINDOW (window->impl);
   if (offscreen->embedder != NULL)
     {
-      gdk_window_get_device_position_double (offscreen->embedder, device, &tmpx, &tmpy, &tmpmask);
+      gdk_window_get_device_position (offscreen->embedder, device, &tmpx, &tmpy, &tmpmask);
       from_embedder (window,
 		     tmpx, tmpy,
 		     &dtmpx, &dtmpy);
-      tmpx = dtmpx;
-      tmpy = dtmpy;
+      tmpx = floor (dtmpx + 0.5);
+      tmpy = floor (dtmpy + 0.5);
     }
 
   if (x)
-    *x = round (tmpx);
+    *x = tmpx;
   if (y)
-    *y = round (tmpy);
+    *y = tmpy;
   if (mask)
     *mask = tmpmask;
   return TRUE;
@@ -327,8 +323,7 @@ gdk_offscreen_window_get_device_state (GdkWindow       *window,
  * If you need to keep this around over window resizes, you need to
  * add a reference to it.
  *
- * Returns: (nullable) (transfer none): The offscreen surface, or
- *   %NULL if not offscreen
+ * Returns: (transfer none): The offscreen surface, or %NULL if not offscreen
  */
 cairo_surface_t *
 gdk_offscreen_window_get_surface (GdkWindow *window)
@@ -521,6 +516,13 @@ gdk_offscreen_window_input_shape_combine_region (GdkWindow       *window,
 {
 }
 
+static gboolean
+gdk_offscreen_window_set_static_gravities (GdkWindow *window,
+					   gboolean   use_static)
+{
+  return TRUE;
+}
+
 static void
 gdk_offscreen_window_get_geometry (GdkWindow *window,
 				   gint      *x,
@@ -541,10 +543,59 @@ gdk_offscreen_window_get_geometry (GdkWindow *window,
     }
 }
 
-static void
+static gboolean
 gdk_offscreen_window_queue_antiexpose (GdkWindow *window,
 				       cairo_region_t *area)
 {
+  return FALSE;
+}
+
+static void
+gdk_offscreen_window_translate (GdkWindow      *window,
+                                cairo_region_t *area,
+                                gint            dx,
+                                gint            dy)
+{
+  GdkOffscreenWindow *offscreen = GDK_OFFSCREEN_WINDOW (window->impl);
+
+  if (offscreen->surface)
+    {
+      cairo_t *cr;
+
+      cr = cairo_create (offscreen->surface);
+
+      area = cairo_region_copy (area);
+
+      gdk_cairo_region (cr, area);
+      cairo_clip (cr);
+
+      /* NB: This is a self-copy and Cairo doesn't support that yet.
+       * So we do a litle trick.
+       */
+      cairo_push_group (cr);
+
+      cairo_set_source_surface (cr, offscreen->surface, dx, dy);
+      cairo_paint (cr);
+
+      cairo_pop_group_to_source (cr);
+      cairo_paint (cr);
+
+      cairo_destroy (cr);
+    }
+
+  _gdk_window_add_damage (window, area);
+}
+
+static cairo_surface_t *
+gdk_offscreen_window_resize_cairo_surface (GdkWindow       *window,
+                                           cairo_surface_t *surface,
+                                           gint             width,
+                                           gint             height)
+{
+  /* No-op.  The surface gets resized in
+   * gdk_offscreen_window_move_resize_internal().
+   */
+  return surface;
 }
 
 /**
@@ -595,8 +646,8 @@ gdk_offscreen_window_set_embedder (GdkWindow     *window,
  *
  * Gets the window that @window is embedded in.
  *
- * Returns: (nullable) (transfer none): the embedding #GdkWindow, or
- *     %NULL if @window is not an mbedded offscreen window
+ * Returns: (transfer none): the embedding #GdkWindow, or %NULL
+ *     if @window is not an mbedded offscreen window
  *
  * Since: 2.18
  */
@@ -644,15 +695,6 @@ gdk_offscreen_window_set_wmfunctions (GdkWindow	    *window,
 {
 }
 
-static void
-gdk_offscreen_window_begin_move_drag (GdkWindow *window,
-                                      gint       button,
-                                      gint       root_x,
-                                      gint       root_y,
-                                      guint32    timestamp)
-{
-}
-
 static void
 gdk_offscreen_window_set_transient_for (GdkWindow *window,
 					GdkWindow *another)
@@ -660,39 +702,10 @@ gdk_offscreen_window_set_transient_for (GdkWindow *window,
 }
 
 static void
-gdk_offscreen_window_get_frame_extents (GdkWindow    *window,
-					GdkRectangle *rect)
+gdk_offscreen_window_process_updates_recurse (GdkWindow *window,
+                                              cairo_region_t *region)
 {
-  rect->x = window->x;
-  rect->y = window->y;
-  rect->width = window->width;
-  rect->height = window->height;
-}
-
-static gint
-gdk_offscreen_window_get_scale_factor (GdkWindow *window)
-{
-  GdkOffscreenWindow *offscreen;
-
-  if (GDK_WINDOW_DESTROYED (window))
-    return 1;
-
-  offscreen = GDK_OFFSCREEN_WINDOW (window->impl);
-  if (offscreen->embedder)
-    return gdk_window_get_scale_factor (offscreen->embedder);
-
-  return gdk_window_get_scale_factor (window->parent);
-}
-
-static void
-gdk_offscreen_window_set_opacity (GdkWindow *window, gdouble opacity)
-{
-}
-
-static gboolean
-gdk_offscreen_window_beep (GdkWindow *window)
-{
-  return FALSE;
+  _gdk_window_process_updates_recurse (window, region);
 }
 
 static void
@@ -716,23 +729,26 @@ gdk_offscreen_window_class_init (GdkOffscreenWindowClass *klass)
   impl_class->move_resize = gdk_offscreen_window_move_resize;
   impl_class->set_background = gdk_offscreen_window_set_background;
   impl_class->reparent = gdk_offscreen_window_reparent;
-  impl_class->set_device_cursor = gdk_offscreen_window_set_device_cursor;
+  impl_class->set_device_cursor = NULL;
   impl_class->get_geometry = gdk_offscreen_window_get_geometry;
   impl_class->get_root_coords = gdk_offscreen_window_get_root_coords;
   impl_class->get_device_state = gdk_offscreen_window_get_device_state;
   impl_class->shape_combine_region = gdk_offscreen_window_shape_combine_region;
   impl_class->input_shape_combine_region = gdk_offscreen_window_input_shape_combine_region;
+  impl_class->set_static_gravities = gdk_offscreen_window_set_static_gravities;
   impl_class->queue_antiexpose = gdk_offscreen_window_queue_antiexpose;
+  impl_class->translate = gdk_offscreen_window_translate;
   impl_class->destroy = gdk_offscreen_window_destroy;
   impl_class->destroy_foreign = NULL;
+  impl_class->resize_cairo_surface = gdk_offscreen_window_resize_cairo_surface;
   impl_class->get_shape = NULL;
   impl_class->get_input_shape = NULL;
-  impl_class->beep = gdk_offscreen_window_beep;
+  impl_class->beep = NULL;
 
   impl_class->focus = NULL;
   impl_class->set_type_hint = NULL;
   impl_class->get_type_hint = NULL;
-  impl_class->set_modal_hint = gdk_offscreen_window_set_boolean;
+  impl_class->set_modal_hint = NULL;
   impl_class->set_skip_taskbar_hint = gdk_offscreen_window_set_boolean;
   impl_class->set_skip_pager_hint = gdk_offscreen_window_set_boolean;
   impl_class->set_urgency_hint = gdk_offscreen_window_set_boolean;
@@ -741,9 +757,10 @@ gdk_offscreen_window_class_init (GdkOffscreenWindowClass *klass)
   impl_class->set_role = gdk_offscreen_window_set_string;
   impl_class->set_startup_id = gdk_offscreen_window_set_string;
   impl_class->set_transient_for = gdk_offscreen_window_set_transient_for;
-  impl_class->get_frame_extents = gdk_offscreen_window_get_frame_extents;
+  impl_class->get_root_origin = NULL;
+  impl_class->get_frame_extents = NULL;
   impl_class->set_override_redirect = NULL;
-  impl_class->set_accept_focus = gdk_offscreen_window_set_boolean;
+  impl_class->set_accept_focus = NULL;
   impl_class->set_focus_on_map = gdk_offscreen_window_set_boolean;
   impl_class->set_icon_list = gdk_offscreen_window_set_list;
   impl_class->set_icon_name = gdk_offscreen_window_set_string;
@@ -763,19 +780,19 @@ gdk_offscreen_window_class_init (GdkOffscreenWindowClass *klass)
   impl_class->get_decorations = NULL;
   impl_class->set_functions = gdk_offscreen_window_set_wmfunctions;
   impl_class->begin_resize_drag = NULL;
-  impl_class->begin_move_drag = gdk_offscreen_window_begin_move_drag;
+  impl_class->begin_move_drag = NULL;
   impl_class->enable_synchronized_configure = gdk_offscreen_window_do_nothing;
   impl_class->configure_finished = NULL;
-  impl_class->set_opacity = gdk_offscreen_window_set_opacity;
+  impl_class->set_opacity = NULL;
   impl_class->set_composited = NULL;
   impl_class->destroy_notify = NULL;
   impl_class->register_dnd = gdk_offscreen_window_do_nothing;
   impl_class->drag_begin = NULL;
+  impl_class->process_updates_recurse = gdk_offscreen_window_process_updates_recurse;
   impl_class->sync_rendering = NULL;
   impl_class->simulate_key = NULL;
   impl_class->simulate_button = NULL;
   impl_class->get_property = NULL;
   impl_class->change_property = NULL;
   impl_class->delete_property = NULL;
-  impl_class->get_scale_factor = gdk_offscreen_window_get_scale_factor;
 }
