@@ -13,7 +13,9 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library. If not, see <http://www.gnu.org/licenses/>.
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
 #include "config.h"
@@ -23,8 +25,6 @@
 #include "gdkcursorprivate.h"
 #include "gdkquartzcursor.h"
 #include "gdkprivate-quartz.h"
-#include "gdkinternal-quartz.h"
-#include "gdkquartz-gtk-only.h"
 
 #include "xcursors.h"
 
@@ -252,169 +252,41 @@ _gdk_quartz_display_get_cursor_for_type (GdkDisplay    *display,
 
 
 GdkCursor *
-_gdk_quartz_display_get_cursor_for_surface (GdkDisplay      *display,
-					    cairo_surface_t *surface,
-					    gdouble          x,
-					    gdouble          y)
+_gdk_quartz_display_get_cursor_for_pixbuf (GdkDisplay *display,
+                                           GdkPixbuf  *pixbuf,
+                                           gint        x,
+                                           gint        y)
 {
   NSImage *image;
   NSCursor *nscursor;
   GdkCursor *cursor;
-  GdkPixbuf *pixbuf;
+  gboolean has_alpha;
 
   GDK_QUARTZ_ALLOC_POOL;
 
-  pixbuf = gdk_pixbuf_get_from_surface (surface, 0, 0,
-					cairo_image_surface_get_width (surface),
-					cairo_image_surface_get_height (surface));
+  has_alpha = gdk_pixbuf_get_has_alpha (pixbuf);
+
   image = gdk_quartz_pixbuf_to_ns_image_libgtk_only (pixbuf);
   nscursor = [[NSCursor alloc] initWithImage:image hotSpot:NSMakePoint(x, y)];
 
   cursor = gdk_quartz_cursor_new_from_nscursor (nscursor, GDK_CURSOR_IS_PIXMAP);
 
-  g_object_unref (pixbuf);
-
   GDK_QUARTZ_RELEASE_POOL;
 
   return cursor;
 }
 
-/* OS X only exports a number of cursor types in its public NSCursor interface.
- * By overriding the private _coreCursorType method, we can tell OS X to load
- * one of its internal cursors instead (since cursor images are loaded on demand
- * instead of in advance). WebKit does this too.
- */
-
-@interface gdkCoreCursor : NSCursor {
-@private
-	int type;
-	BOOL override;
-}
-@end
-
-@implementation gdkCoreCursor
-
-- (int)_coreCursorType
-{
-	if (self->override)
-		return self->type;
-	return [super _coreCursorType];
-}
-
-#define CUSTOM_CURSOR_CTOR(name, id) \
-	+ (gdkCoreCursor *)name \
-	{ \
-		gdkCoreCursor *obj; \
-		obj = [self new]; \
-		if (obj) { \
-			obj->override = YES; \
-			obj->type = id; \
-		} \
-		return obj; \
-	}
-CUSTOM_CURSOR_CTOR(gdkHelpCursor, 40)
-CUSTOM_CURSOR_CTOR(gdkProgressCursor, 4)
-/* TODO OS X doesn't seem to have a way to get this. There is an undocumented
- * method +[NSCursor _waitCursor], but it doesn't actually return this cursor,
- * but rather some odd low-quality non-animating version of this cursor. Use
- * the progress cursor instead for now.
- */
-CUSTOM_CURSOR_CTOR(gdkWaitCursor, 4)
-CUSTOM_CURSOR_CTOR(gdkAliasCursor, 2)
-CUSTOM_CURSOR_CTOR(gdkMoveCursor, 39)
-/* TODO OS X doesn't seem to provide one; copy the move cursor for now
- *  since it looks similar to what we want. */
-CUSTOM_CURSOR_CTOR(gdkAllScrollCursor, 39)
-CUSTOM_CURSOR_CTOR(gdkNEResizeCursor, 29)
-CUSTOM_CURSOR_CTOR(gdkNWResizeCursor, 33)
-CUSTOM_CURSOR_CTOR(gdkSEResizeCursor, 35)
-CUSTOM_CURSOR_CTOR(gdkSWResizeCursor, 37)
-CUSTOM_CURSOR_CTOR(gdkEWResizeCursor, 28)
-CUSTOM_CURSOR_CTOR(gdkNSResizeCursor, 32)
-CUSTOM_CURSOR_CTOR(gdkNESWResizeCursor, 30)
-CUSTOM_CURSOR_CTOR(gdkNWSEResizeCursor, 34)
-CUSTOM_CURSOR_CTOR(gdkZoomInCursor, 42)
-CUSTOM_CURSOR_CTOR(gdkZoomOutCursor, 43)
-
-@end
-
-struct CursorsByName {
-  const gchar *name;
-  NSString *selector;
-};
-
-static const struct CursorsByName cursors_by_name[] = {
-  /* Link & Status */
-  { "context-menu", @"contextualMenuCursor" },
-  { "help", @"gdkHelpCursor" },
-  { "pointer", @"pointingHandCursor" },
-  { "progress", @"gdkProgressCursor" },
-  { "wait", @"gdkWaitCursor" },
-  /* Selection */
-  { "cell", @"crosshairCursor" },
-  { "crosshair", @"crosshairCursor" },
-  { "text", @"IBeamCursor" },
-  { "vertical-text", @"IBeamCursorForVerticalLayout" },
-  /* Drag & Drop */
-  { "alias", @"gdkAliasCursor" },
-  { "copy", @"dragCopyCursor" },
-  { "move", @"gdkMoveCursor" },
-  { "no-drop", @"operationNotAllowedCursor" },
-  { "not-allowed", @"operationNotAllowedCursor" },
-  { "grab", @"openHandCursor" },
-  { "grabbing", @"closedHandCursor" },
-  /* Resize & Scrolling */
-  { "all-scroll", @"gdkAllScrollCursor" },
-  { "col-resize", @"resizeLeftRightCursor" },
-  { "row-resize", @"resizeUpDownCursor" },
-  { "n-resize", @"resizeUpCursor" },
-  { "e-resize", @"resizeRightCursor" },
-  { "s-resize", @"resizeDownCursor" },
-  { "w-resize", @"resizeLeftCursor" },
-  { "ne-resize", @"gdkNEResizeCursor" },
-  { "nw-resize", @"gdkNWResizeCursor" },
-  { "se-resize", @"gdkSEResizeCursor" },
-  { "sw-resize", @"gdkSWResizeCursor" },
-  { "ew-resize", @"gdkEWResizeCursor" },
-  { "ns-resize", @"gdkNSResizeCursor" },
-  { "nesw-resize", @"gdkNESWResizeCursor" },
-  { "nwse-resize", @"gdkNWSEResizeCursor" },
-  /* Zoom */
-  { "zoom-in", @"gdkZoomInCursor" },
-  { "zoom-out", @"gdkZoomOutCursor" },
-  { NULL, NULL },
-};
-
 GdkCursor*
 _gdk_quartz_display_get_cursor_for_name (GdkDisplay  *display,
                                          const gchar *name)
 {
-  NSCursor *nscursor;
-  const struct CursorsByName *test;
-  SEL selector;
-
-  if (name == NULL || g_str_equal (name, "none"))
-    return create_blank_cursor ();
-
-  // use this selector if nothing found
-  selector = @selector(arrowCursor);
-  for (test = cursors_by_name; test->name != NULL; test++)
-    if (g_str_equal (name, test->name))
-      {
-        selector = NSSelectorFromString(test->selector);
-        break;
-      }
-  nscursor = [[gdkCoreCursor class] performSelector:selector];
-
-  [nscursor retain];
-  return gdk_quartz_cursor_new_from_nscursor (nscursor, GDK_CURSOR_IS_PIXMAP);
+  /* FIXME: Implement */
+  return NULL;
 }
 
 G_DEFINE_TYPE (GdkQuartzCursor, gdk_quartz_cursor, GDK_TYPE_CURSOR)
 
-static cairo_surface_t *gdk_quartz_cursor_get_surface (GdkCursor *cursor,
-						       gdouble *x_hot,
-						       gdouble *y_hot);
+static GdkPixbuf *gdk_quartz_cursor_get_image (GdkCursor *cursor);
 
 static void
 gdk_quartz_cursor_finalize (GObject *object)
@@ -434,7 +306,7 @@ gdk_quartz_cursor_class_init (GdkQuartzCursorClass *quartz_cursor_class)
 
   object_class->finalize = gdk_quartz_cursor_finalize;
 
-  cursor_class->get_surface = gdk_quartz_cursor_get_surface;
+  cursor_class->get_image = gdk_quartz_cursor_get_image;
 }
 
 static void
@@ -490,10 +362,8 @@ _gdk_quartz_cursor_get_ns_cursor (GdkCursor *cursor)
   return cursor_private->nscursor;
 }
 
-static cairo_surface_t *
-gdk_quartz_cursor_get_surface (GdkCursor *cursor,
-			       gdouble *x_hot,
-			       gdouble *y_hot)
+static GdkPixbuf *
+gdk_quartz_cursor_get_image (GdkCursor *cursor)
 {
   /* FIXME: Implement */
   return NULL;
