@@ -136,7 +136,7 @@ const static struct {
   guint keycode;
   guint keyval;
   unsigned int modmask; /* So we can tell when a mod key is pressed/released */
-} modifier_keys[] = {
+} known_keys[] = {
   {  54, GDK_KEY_Meta_R,    NSCommandKeyMask },
   {  55, GDK_KEY_Meta_L,    NSCommandKeyMask },
   {  56, GDK_KEY_Shift_L,   NSShiftKeyMask },
@@ -145,29 +145,23 @@ const static struct {
   {  59, GDK_KEY_Control_L, NSControlKeyMask },
   {  60, GDK_KEY_Shift_R,   NSShiftKeyMask },
   {  61, GDK_KEY_Alt_R,     NSAlternateKeyMask },
-  {  62, GDK_KEY_Control_R, NSControlKeyMask }
-};
-
-const static struct {
-  guint keycode;
-  guint keyval;
-} function_keys[] = {
-  { 122, GDK_KEY_F1 },
-  { 120, GDK_KEY_F2 },
-  {  99, GDK_KEY_F3 },
-  { 118, GDK_KEY_F4 },
-  {  96, GDK_KEY_F5 },
-  {  97, GDK_KEY_F6 },
-  {  98, GDK_KEY_F7 },
-  { 100, GDK_KEY_F8 },
-  { 101, GDK_KEY_F9 },
-  { 109, GDK_KEY_F10 },
-  { 103, GDK_KEY_F11 },
-  { 111, GDK_KEY_F12 },
-  { 105, GDK_KEY_F13 },
-  { 107, GDK_KEY_F14 },
-  { 113, GDK_KEY_F15 },
-  { 106, GDK_KEY_F16 }
+  {  62, GDK_KEY_Control_R, NSControlKeyMask },
+  { 122, GDK_KEY_F1, 0 },
+  { 120, GDK_KEY_F2, 0 },
+  {  99, GDK_KEY_F3, 0 },
+  { 118, GDK_KEY_F4, 0 },
+  {  96, GDK_KEY_F5, 0 },
+  {  97, GDK_KEY_F6, 0 },
+  {  98, GDK_KEY_F7, 0 },
+  { 100, GDK_KEY_F8, 0 },
+  { 101, GDK_KEY_F9, 0 },
+  { 109, GDK_KEY_F10, 0 },
+  { 103, GDK_KEY_F11, 0 },
+  { 111, GDK_KEY_F12, 0 },
+  { 105, GDK_KEY_F13, 0 },
+  { 107, GDK_KEY_F14, 0 },
+  { 113, GDK_KEY_F15, 0 },
+  { 106, GDK_KEY_F16, 0 }
 };
 
 const static struct {
@@ -410,6 +404,7 @@ maybe_update_keymap (void)
 		                        0,
 		                        &state, 4, &nChars, chars);
 
+
                   /* FIXME: Theoretically, we can get multiple UTF-16 values;
 		   * we should convert them to proper unicode and figure
 		   * out whether there are really keyboard layouts that
@@ -475,21 +470,13 @@ maybe_update_keymap (void)
 	}
 #endif
 
-      for (i = 0; i < G_N_ELEMENTS (modifier_keys); i++)
+      for (i = 0; i < G_N_ELEMENTS (known_keys); i++)
 	{
-	  p = keyval_array + modifier_keys[i].keycode * KEYVALS_PER_KEYCODE;
+	  p = keyval_array + known_keys[i].keycode * KEYVALS_PER_KEYCODE;
 
 	  if (p[0] == 0 && p[1] == 0 && 
 	      p[2] == 0 && p[3] == 0)
-	    p[0] = modifier_keys[i].keyval;
-	}
-
-      for (i = 0; i < G_N_ELEMENTS (function_keys); i++)
-	{
-	  p = keyval_array + function_keys[i].keycode * KEYVALS_PER_KEYCODE;
-
-          p[0] = function_keys[i].keyval;
-          p[1] = p[2] = p[3] = 0;
+	    p[0] = known_keys[i].keyval;
 	}
 
       for (i = 0; i < G_N_ELEMENTS (known_numeric_keys); i++)
@@ -749,6 +736,35 @@ gdk_quartz_keymap_map_virtual_modifiers (GdkKeymap       *keymap,
   return TRUE;
 }
 
+static GdkModifierType
+gdk_quartz_keymap_get_modifier_mask (GdkKeymap         *keymap,
+                                     GdkModifierIntent  intent)
+{
+  switch (intent)
+    {
+    case GDK_MODIFIER_INTENT_PRIMARY_ACCELERATOR:
+      return GDK_MOD2_MASK;
+
+    case GDK_MODIFIER_INTENT_CONTEXT_MENU:
+      return GDK_CONTROL_MASK;
+
+    case GDK_MODIFIER_INTENT_EXTEND_SELECTION:
+      return GDK_SHIFT_MASK;
+
+    case GDK_MODIFIER_INTENT_MODIFY_SELECTION:
+      return GDK_MOD2_MASK;
+
+    case GDK_MODIFIER_INTENT_NO_TEXT_INPUT:
+      return GDK_MOD2_MASK | GDK_CONTROL_MASK;
+
+    case GDK_MODIFIER_INTENT_SHIFT_GROUP:
+      return GDK_MOD1_MASK;
+
+    default:
+      g_return_val_if_reached (0);
+    }
+}
+
 /* What sort of key event is this? Returns one of
  * GDK_KEY_PRESS, GDK_KEY_RELEASE, GDK_NOTHING (should be ignored)
  */
@@ -776,11 +792,11 @@ _gdk_quartz_keys_event_type (NSEvent *event)
   keycode = [event keyCode];
   flags = [event modifierFlags];
   
-  for (i = 0; i < G_N_ELEMENTS (modifier_keys); i++)
+  for (i = 0; i < G_N_ELEMENTS (known_keys); i++)
     {
-      if (modifier_keys[i].keycode == keycode)
+      if (known_keys[i].keycode == keycode)
 	{
-	  if (flags & modifier_keys[i].modmask)
+	  if (flags & known_keys[i].modmask)
 	    return GDK_KEY_PRESS;
 	  else
 	    return GDK_KEY_RELEASE;
@@ -797,12 +813,12 @@ _gdk_quartz_keys_is_modifier (guint keycode)
 {
   gint i;
   
-  for (i = 0; i < G_N_ELEMENTS (modifier_keys); i++)
+  for (i = 0; i < G_N_ELEMENTS (known_keys); i++)
     {
-      if (modifier_keys[i].modmask == 0)
+      if (known_keys[i].modmask == 0)
 	break;
 
-      if (modifier_keys[i].keycode == keycode)
+      if (known_keys[i].keycode == keycode)
 	return TRUE;
     }
 
@@ -838,4 +854,5 @@ gdk_quartz_keymap_class_init (GdkQuartzKeymapClass *klass)
   keymap_class->translate_keyboard_state = gdk_quartz_keymap_translate_keyboard_state;
   keymap_class->add_virtual_modifiers = gdk_quartz_keymap_add_virtual_modifiers;
   keymap_class->map_virtual_modifiers = gdk_quartz_keymap_map_virtual_modifiers;
+  keymap_class->get_modifier_mask = gdk_quartz_keymap_get_modifier_mask;
 }
