@@ -1,4 +1,5 @@
-/* GTK+ - accessibility implementations
+/* GAIL - The GNOME Accessibility Implementation Library
+
  * Copyright 2001 Sun Microsystems Inc.
  *
  * This library is free software; you can redistribute it and/or
@@ -12,7 +13,9 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library. If not, see <http://www.gnu.org/licenses/>.
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
 #include "config.h"
@@ -21,7 +24,7 @@
 #include "gtkcellaccessibleparent.h"
 
 GType
-gtk_cell_accessible_parent_get_type (void)
+_gtk_cell_accessible_parent_get_type (void)
 {
   static volatile gsize g_define_type_id__volatile = 0;
 
@@ -42,21 +45,14 @@ gtk_cell_accessible_parent_get_type (void)
   return g_define_type_id__volatile;
 }
 
-/**
- * gtk_cell_accessible_parent_get_cell_extents:
- * @x: (out):
- * @y: (out):
- * @width: (out):
- * @height: (out):
- */
 void
-gtk_cell_accessible_parent_get_cell_extents (GtkCellAccessibleParent *parent,
-                                             GtkCellAccessible       *cell,
-                                             gint                    *x,
-                                             gint                    *y,
-                                             gint                    *width,
-                                             gint                    *height,
-                                             AtkCoordType             coord_type)
+_gtk_cell_accessible_parent_get_cell_extents (GtkCellAccessibleParent *parent,
+                                              GtkCellAccessible       *cell,
+                                              gint                    *x,
+                                              gint                    *y,
+                                              gint                    *width,
+                                              gint                    *height,
+                                              AtkCoordType             coord_type)
 {
   GtkCellAccessibleParentIface *iface;
 
@@ -68,14 +64,10 @@ gtk_cell_accessible_parent_get_cell_extents (GtkCellAccessibleParent *parent,
     (iface->get_cell_extents) (parent, cell, x, y, width, height, coord_type);
 }
 
-/**
- * gtk_cell_accessible_parent_get_cell_area:
- * @cell_rect: (out):
- */
 void
-gtk_cell_accessible_parent_get_cell_area (GtkCellAccessibleParent *parent,
-                                          GtkCellAccessible       *cell,
-                                          GdkRectangle            *cell_rect)
+_gtk_cell_accessible_parent_get_cell_area (GtkCellAccessibleParent *parent,
+                                           GtkCellAccessible       *cell,
+                                           GdkRectangle            *cell_rect)
 {
   GtkCellAccessibleParentIface *iface;
 
@@ -89,8 +81,8 @@ gtk_cell_accessible_parent_get_cell_area (GtkCellAccessibleParent *parent,
 }
 
 gboolean
-gtk_cell_accessible_parent_grab_focus (GtkCellAccessibleParent *parent,
-                                       GtkCellAccessible       *cell)
+_gtk_cell_accessible_parent_grab_focus (GtkCellAccessibleParent *parent,
+                                        GtkCellAccessible       *cell)
 {
   GtkCellAccessibleParentIface *iface;
 
@@ -105,8 +97,8 @@ gtk_cell_accessible_parent_grab_focus (GtkCellAccessibleParent *parent,
 }
 
 int
-gtk_cell_accessible_parent_get_child_index (GtkCellAccessibleParent *parent,
-                                            GtkCellAccessible       *cell)
+_gtk_cell_accessible_parent_get_child_index (GtkCellAccessibleParent *parent,
+                                             GtkCellAccessible       *cell)
 {
   GtkCellAccessibleParentIface *iface;
 
@@ -121,8 +113,8 @@ gtk_cell_accessible_parent_get_child_index (GtkCellAccessibleParent *parent,
 }
 
 GtkCellRendererState
-gtk_cell_accessible_parent_get_renderer_state (GtkCellAccessibleParent *parent,
-                                               GtkCellAccessible       *cell)
+_gtk_cell_accessible_parent_get_renderer_state (GtkCellAccessibleParent *parent,
+                                                GtkCellAccessible       *cell)
 {
   GtkCellAccessibleParentIface *iface;
 
@@ -138,8 +130,8 @@ gtk_cell_accessible_parent_get_renderer_state (GtkCellAccessibleParent *parent,
 }
 
 void
-gtk_cell_accessible_parent_expand_collapse (GtkCellAccessibleParent *parent,
-                                            GtkCellAccessible       *cell)
+_gtk_cell_accessible_parent_expand_collapse (GtkCellAccessibleParent *parent,
+                                             GtkCellAccessible       *cell)
 {
   GtkCellAccessibleParentIface *iface;
 
@@ -153,8 +145,8 @@ gtk_cell_accessible_parent_expand_collapse (GtkCellAccessibleParent *parent,
 }
 
 void
-gtk_cell_accessible_parent_activate (GtkCellAccessibleParent *parent,
-                                     GtkCellAccessible       *cell)
+_gtk_cell_accessible_parent_activate (GtkCellAccessibleParent *parent,
+                                      GtkCellAccessible       *cell)
 {
   GtkCellAccessibleParentIface *iface;
 
@@ -168,8 +160,8 @@ gtk_cell_accessible_parent_activate (GtkCellAccessibleParent *parent,
 }
 
 void
-gtk_cell_accessible_parent_edit (GtkCellAccessibleParent *parent,
-                                 GtkCellAccessible       *cell)
+_gtk_cell_accessible_parent_edit (GtkCellAccessibleParent *parent,
+                                  GtkCellAccessible       *cell)
 {
   GtkCellAccessibleParentIface *iface;
 
@@ -181,91 +173,3 @@ gtk_cell_accessible_parent_edit (GtkCellAccessibleParent *parent,
   if (iface->edit)
     (iface->edit) (parent, cell);
 }
-
-void
-gtk_cell_accessible_parent_update_relationset (GtkCellAccessibleParent *parent,
-                                               GtkCellAccessible       *cell,
-                                               AtkRelationSet          *relationset)
-{
-  GtkCellAccessibleParentIface *iface;
-
-  g_return_if_fail (GTK_IS_CELL_ACCESSIBLE_PARENT (parent));
-  g_return_if_fail (GTK_IS_CELL_ACCESSIBLE (cell));
-  g_return_if_fail (ATK_IS_RELATION_SET (relationset));
-
-  iface = GTK_CELL_ACCESSIBLE_PARENT_GET_IFACE (parent);
-
-  if (iface->update_relationset)
-    (iface->update_relationset) (parent, cell, relationset);
-}
-
-/**
- * gtk_cell_accessible_parent_get_cell_position:
- * @row: (out):
- * @column: (out):
- */
-void
-gtk_cell_accessible_parent_get_cell_position (GtkCellAccessibleParent *parent,
-                                              GtkCellAccessible       *cell,
-                                              gint                    *row,
-                                              gint                    *column)
-{
-  GtkCellAccessibleParentIface *iface;
-
-  g_return_if_fail (GTK_IS_CELL_ACCESSIBLE_PARENT (parent));
-  g_return_if_fail (GTK_IS_CELL_ACCESSIBLE (cell));
-
-  iface = GTK_CELL_ACCESSIBLE_PARENT_GET_IFACE (parent);
-
-  if (iface->get_cell_position)
-    (iface->get_cell_position) (parent, cell, row, column);
-  else
-    {
-      if (row)
-        *row = -1;
-      if (column)
-        *column = -1;
-    }
-}
-
-/**
- * gtk_cell_accessible_parent_get_column_header_cells:
- * Returns: (transfer full) (element-type AtkObject)
- */
-GPtrArray *
-gtk_cell_accessible_parent_get_column_header_cells (GtkCellAccessibleParent *parent,
-                                                    GtkCellAccessible       *cell)
-{
-  GtkCellAccessibleParentIface *iface;
-
-  g_return_val_if_fail (GTK_IS_CELL_ACCESSIBLE_PARENT (parent), NULL);
-  g_return_val_if_fail (GTK_IS_CELL_ACCESSIBLE (cell), NULL);
-
-  iface = GTK_CELL_ACCESSIBLE_PARENT_GET_IFACE (parent);
-
-  if (iface->get_column_header_cells)
-    return (iface->get_column_header_cells) (parent, cell);
-  else
-    return NULL;
-}
-
-/**
- * gtk_cell_accessible_parent_get_row_header_cells:
- * Returns: (transfer full) (element-type AtkObject)
- */
-GPtrArray *
-gtk_cell_accessible_parent_get_row_header_cells (GtkCellAccessibleParent *parent,
-                                                    GtkCellAccessible       *cell)
-{
-  GtkCellAccessibleParentIface *iface;
-
-  g_return_val_if_fail (GTK_IS_CELL_ACCESSIBLE_PARENT (parent), NULL);
-  g_return_val_if_fail (GTK_IS_CELL_ACCESSIBLE (cell), NULL);
-
-  iface = GTK_CELL_ACCESSIBLE_PARENT_GET_IFACE (parent);
-
-  if (iface->get_row_header_cells)
-    return (iface->get_row_header_cells) (parent, cell);
-  else
-    return NULL;
-}
