@@ -12,7 +12,9 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library. If not, see <http://www.gnu.org/licenses/>.
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
 /*
@@ -22,13 +24,13 @@
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/.
  */
 
-#ifndef __GTK_COLOR_SELECTION_DIALOG_H__
-#define __GTK_COLOR_SELECTION_DIALOG_H__
-
 #if !defined (__GTK_H_INSIDE__) && !defined (GTK_COMPILATION)
 #error "Only <gtk/gtk.h> can be included directly."
 #endif
 
+#ifndef __GTK_COLOR_SELECTION_DIALOG_H__
+#define __GTK_COLOR_SELECTION_DIALOG_H__
+
 #include <gtk/gtkdialog.h>
 #include <gtk/deprecated/gtkcolorsel.h>
 
@@ -68,11 +70,10 @@ struct _GtkColorSelectionDialogClass
 
 
 /* ColorSelectionDialog */
-GDK_DEPRECATED_IN_3_4
 GType      gtk_color_selection_dialog_get_type            (void) G_GNUC_CONST;
-GDK_DEPRECATED_IN_3_4_FOR(gtk_color_chooser_dialog_new)
+GDK_DEPRECATED_FOR(gtk_color_chooser_dialog_new)
 GtkWidget* gtk_color_selection_dialog_new                 (const gchar *title);
-GDK_DEPRECATED_IN_3_4_FOR(GtkColorChooser)
+GDK_DEPRECATED_FOR(GtkColorChooser)
 GtkWidget* gtk_color_selection_dialog_get_color_selection (GtkColorSelectionDialog *colorsel);
 
 
