@@ -17,7 +17,9 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library. If not, see <http://www.gnu.org/licenses/>.
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
 /*
@@ -53,6 +55,9 @@
 #include "gtkbuildable.h"
 #include "gtkorientable.h"
 #include "gtkprivate.h"
+
+#define GDK_DISABLE_DEPRECATION_WARNINGS
+
 #include "gtkfontsel.h"
 
 /**
@@ -219,7 +224,7 @@ static void     gtk_font_selection_ref_family            (GtkFontSelection *font
 static void     gtk_font_selection_ref_face              (GtkFontSelection *fontsel,
 							  PangoFontFace    *face);
 
-G_DEFINE_TYPE_WITH_PRIVATE (GtkFontSelection, gtk_font_selection, GTK_TYPE_BOX)
+G_DEFINE_TYPE (GtkFontSelection, gtk_font_selection, GTK_TYPE_BOX)
 
 static void
 gtk_font_selection_class_init (GtkFontSelectionClass *klass)
@@ -248,6 +253,8 @@ gtk_font_selection_class_init (GtkFontSelectionClass *klass)
                                                         P_("The text to display in order to demonstrate the selected font"),
                                                         _(PREVIEW_TEXT),
                                                         GTK_PARAM_READWRITE));
+
+  g_type_class_add_private (klass, sizeof (GtkFontSelectionPrivate));
 }
 
 static void 
@@ -337,7 +344,9 @@ gtk_font_selection_init (GtkFontSelection *fontsel)
   GList *focus_chain = NULL;
   AtkObject *atk_obj;
 
-  fontsel->priv = gtk_font_selection_get_instance_private (fontsel);
+  fontsel->priv = G_TYPE_INSTANCE_GET_PRIVATE (fontsel,
+                                               GTK_TYPE_FONT_SELECTION,
+                                               GtkFontSelectionPrivate);
   priv = fontsel->priv;
 
   gtk_orientable_set_orientation (GTK_ORIENTABLE (fontsel),
@@ -603,9 +612,7 @@ gtk_font_selection_init (GtkFontSelection *fontsel)
  *
  * Creates a new #GtkFontSelection.
  *
- * Returns: a new #GtkFontSelection
- *
- * Deprecated: 3.2: Use #GtkFontChooserWidget instead
+ * Return value: a n ew #GtkFontSelection
  */
 GtkWidget *
 gtk_font_selection_new (void)
@@ -816,11 +823,12 @@ gtk_font_selection_show_available_fonts (GtkFontSelection *fontsel)
       const gchar *name = pango_font_family_get_name (families[i]);
       GtkTreeIter iter;
 
-      gtk_list_store_insert_with_values (model, &iter, -1,
-                                         FAMILY_COLUMN, families[i],
-                                         FAMILY_NAME_COLUMN, name,
-                                         -1);
-
+      gtk_list_store_append (model, &iter);
+      gtk_list_store_set (model, &iter,
+			  FAMILY_COLUMN, families[i],
+			  FAMILY_NAME_COLUMN, name,
+			  -1);
+      
       if (i == 0 || !g_ascii_strcasecmp (name, "sans"))
 	{
 	  match_family = families[i];
@@ -917,10 +925,11 @@ gtk_font_selection_show_available_styles (GtkFontSelection *fontsel)
       GtkTreeIter iter;
       const gchar *str = pango_font_face_get_face_name (faces[i]);
 
-      gtk_list_store_insert_with_values (model, &iter, -1,
-                                         FACE_COLUMN, faces[i],
-                                         FACE_NAME_COLUMN, str,
-                                         -1);
+      gtk_list_store_append (model, &iter);
+      gtk_list_store_set (model, &iter,
+			  FACE_COLUMN, faces[i],
+			  FACE_NAME_COLUMN, str,
+			  -1);
 
       if (i == 0)
 	{
@@ -1027,8 +1036,8 @@ gtk_font_selection_show_available_sizes (GtkFontSelection *fontsel,
 	{
 	  GtkTreeIter iter;
 
-	  gtk_list_store_insert_with_values (model, &iter, -1,
-	                                     SIZE_COLUMN, font_sizes[i], -1);
+	  gtk_list_store_append (model, &iter);
+	  gtk_list_store_set (model, &iter, SIZE_COLUMN, font_sizes[i], -1);
 
 	  if (font_sizes[i] * PANGO_SCALE == priv->size)
 	    set_cursor_to_iter (GTK_TREE_VIEW (priv->size_list), &iter);
@@ -1048,8 +1057,7 @@ gtk_font_selection_show_available_sizes (GtkFontSelection *fontsel,
 	      found = TRUE;
 	    }
 
-          if (!gtk_tree_model_iter_next (GTK_TREE_MODEL (model), &iter))
-            break;
+	  gtk_tree_model_iter_next (GTK_TREE_MODEL (model), &iter);
 	}
 
       if (!found)
@@ -1111,7 +1119,7 @@ gtk_font_selection_size_activate (GtkWidget   *w,
   const gchar *text;
 
   text = gtk_entry_get_text (GTK_ENTRY (priv->size_entry));
-  new_size = (int) MAX (0.1, atof (text) * PANGO_SCALE + 0.5);
+  new_size = MAX (0.1, atof (text) * PANGO_SCALE + 0.5);
 
   if (priv->size != new_size)
     gtk_font_selection_set_size (fontsel, new_size);
@@ -1130,7 +1138,7 @@ gtk_font_selection_size_focus_out (GtkWidget     *w,
   const gchar *text;
 
   text = gtk_entry_get_text (GTK_ENTRY (priv->size_entry));
-  new_size = (int) MAX (0.1, atof (text) * PANGO_SCALE + 0.5);
+  new_size = MAX (0.1, atof (text) * PANGO_SCALE + 0.5);
 
   gtk_font_selection_set_size (fontsel, new_size);
 
@@ -1177,18 +1185,36 @@ gtk_font_selection_get_font_description (GtkFontSelection *fontsel)
   return font_desc;
 }
 
-/* This sets the font in the preview entry to the selected font.
+/* This sets the font in the preview entry to the selected font,
+ * and tries to make sure that the preview entry is a reasonable
+ * size, i.e. so that the text can be seen with a bit of space to
+ * spare. But it tries to avoid resizing the entry every time the
+ * font changes. This also used to shrink the preview if the font
+ * size was decreased, but that made it awkward if the user wanted
+ * to resize the window themself.
  */
 static void
 gtk_font_selection_update_preview (GtkFontSelection *fontsel)
 {
   GtkFontSelectionPrivate *priv = fontsel->priv;
+  gint new_height;
+  GtkRequisition old_requisition, new_requisition;
   GtkWidget *preview_entry = priv->preview_entry;
   const gchar *text;
 
+  gtk_widget_get_preferred_size (preview_entry, &old_requisition, NULL);
+
   gtk_widget_override_font (preview_entry,
                             gtk_font_selection_get_font_description (fontsel));
 
+  gtk_widget_get_preferred_size (preview_entry, &new_requisition, NULL);
+
+  /* We don't ever want to be over MAX_PREVIEW_HEIGHT pixels high. */
+  new_height = CLAMP (new_requisition.height, INITIAL_PREVIEW_HEIGHT, MAX_PREVIEW_HEIGHT);
+
+  if (new_height > old_requisition.height || new_height < old_requisition.height - 30)
+    gtk_widget_set_size_request (preview_entry, -1, new_height);
+
   /* This sets the preview text, if it hasn't been set already. */
   text = gtk_entry_get_text (GTK_ENTRY (preview_entry));
   if (strlen (text) == 0)
@@ -1206,9 +1232,9 @@ gtk_font_selection_update_preview (GtkFontSelection *fontsel)
  * @fontsel: a #GtkFontSelection
  *
  * This returns the #GtkTreeView that lists font families, for
- * example, “Sans”, “Serif”, etc.
+ * example, 'Sans', 'Serif', etc.
  *
- * Returns: (transfer none): A #GtkWidget that is part of @fontsel
+ * Return value: (transfer none): A #GtkWidget that is part of @fontsel
  *
  * Since: 2.14
  *
@@ -1227,9 +1253,9 @@ gtk_font_selection_get_family_list (GtkFontSelection *fontsel)
  * @fontsel: a #GtkFontSelection
  *
  * This returns the #GtkTreeView which lists all styles available for
- * the selected font. For example, “Regular”, “Bold”, etc.
+ * the selected font. For example, 'Regular', 'Bold', etc.
  * 
- * Returns: (transfer none): A #GtkWidget that is part of @fontsel
+ * Return value: (transfer none): A #GtkWidget that is part of @fontsel
  *
  * Since: 2.14
  *
@@ -1250,7 +1276,7 @@ gtk_font_selection_get_face_list (GtkFontSelection *fontsel)
  * This returns the #GtkEntry used to allow the user to edit the font
  * number manually instead of selecting it from the list of font sizes.
  *
- * Returns: (transfer none): A #GtkWidget that is part of @fontsel
+ * Return value: (transfer none): A #GtkWidget that is part of @fontsel
  *
  * Since: 2.14
  *
@@ -1268,9 +1294,9 @@ gtk_font_selection_get_size_entry (GtkFontSelection *fontsel)
  * gtk_font_selection_get_size_list:
  * @fontsel: a #GtkFontSelection
  *
- * This returns the #GtkTreeView used to list font sizes.
+ * This returns the #GtkTreeeView used to list font sizes.
  *
- * Returns: (transfer none): A #GtkWidget that is part of @fontsel
+ * Return value: (transfer none): A #GtkWidget that is part of @fontsel
  *
  * Since: 2.14
  *
@@ -1290,7 +1316,7 @@ gtk_font_selection_get_size_list (GtkFontSelection *fontsel)
  *
  * This returns the #GtkEntry used to display the font as a preview.
  *
- * Returns: (transfer none): A #GtkWidget that is part of @fontsel
+ * Return value: (transfer none): A #GtkWidget that is part of @fontsel
  *
  * Since: 2.14
  *
@@ -1310,7 +1336,7 @@ gtk_font_selection_get_preview_entry (GtkFontSelection *fontsel)
  *
  * Gets the #PangoFontFamily representing the selected font family.
  *
- * Returns: (transfer none): A #PangoFontFamily representing the
+ * Return value: (transfer none): A #PangoFontFamily representing the
  *     selected font family. Font families are a collection of font
  *     faces. The returned object is owned by @fontsel and must not
  *     be modified or freed.
@@ -1334,7 +1360,7 @@ gtk_font_selection_get_family (GtkFontSelection *fontsel)
  * Gets the #PangoFontFace representing the selected font group
  * details (i.e. family, slant, weight, width, etc).
  *
- * Returns: (transfer none): A #PangoFontFace representing the
+ * Return value: (transfer none): A #PangoFontFace representing the
  *     selected font group details. The returned object is owned by
  *     @fontsel and must not be modified or freed.
  *
@@ -1356,7 +1382,7 @@ gtk_font_selection_get_face (GtkFontSelection *fontsel)
  *
  * The selected font size.
  *
- * Returns: A n integer representing the selected font size,
+ * Return value: A n integer representing the selected font size,
  *     or -1 if no font size is selected.
  *
  * Since: 2.14
@@ -1380,11 +1406,11 @@ gtk_font_selection_get_size (GtkFontSelection *fontsel)
  * Note that this can be a different string than what you set with 
  * gtk_font_selection_set_font_name(), as the font selection widget may 
  * normalize font names and thus return a string with a different structure. 
- * For example, “Helvetica Italic Bold 12” could be normalized to 
- * “Helvetica Bold Italic 12”. Use pango_font_description_equal()
+ * For example, "Helvetica Italic Bold 12" could be normalized to 
+ * "Helvetica Bold Italic 12". Use pango_font_description_equal()
  * if you want to compare two font descriptions.
  * 
- * Returns: A string with the name of the current font, or %NULL if 
+ * Return value: A string with the name of the current font, or %NULL if 
  *     no font is selected. You must free this string with g_free().
  *
  * Deprecated: 3.2: Use #GtkFontChooser
@@ -1510,7 +1536,7 @@ gtk_font_selection_select_font_desc (GtkFontSelection      *fontsel,
 /**
  * gtk_font_selection_set_font_name:
  * @fontsel: a #GtkFontSelection
- * @fontname: a font name like “Helvetica 12” or “Times Bold 18”
+ * @fontname: a font name like "Helvetica 12" or "Times Bold 18"
  * 
  * Sets the currently-selected font. 
  *
@@ -1518,8 +1544,8 @@ gtk_font_selection_select_font_desc (GtkFontSelection      *fontsel,
  * for this to work; this can be guaranteed by simply making sure that the 
  * @fontsel is inserted in a toplevel window before you call this function.
  * 
- * Returns: %TRUE if the font could be set successfully; %FALSE if no 
- *     such font exists or if the @fontsel doesn’t belong to a particular 
+ * Return value: %TRUE if the font could be set successfully; %FALSE if no 
+ *     such font exists or if the @fontsel doesn't belong to a particular 
  *     screen yet.
  *
  * Deprecated: 3.2: Use #GtkFontChooser
@@ -1563,7 +1589,7 @@ gtk_font_selection_set_font_name (GtkFontSelection *fontsel,
  *
  * Gets the text displayed in the preview area.
  * 
- * Returns: the text displayed in the preview area. 
+ * Return value: the text displayed in the preview area. 
  *     This string is owned by the widget and should not be 
  *     modified or freed 
  *
@@ -1626,12 +1652,13 @@ gtk_font_selection_set_preview_text  (GtkFontSelection *fontsel,
  * In GTK+ 3.2, #GtkFontSelectionDialog has been deprecated in favor of
  * #GtkFontChooserDialog.
  *
- * # GtkFontSelectionDialog as GtkBuildable # {#GtkFontSelectionDialog-BUILDER-UI}
- *
+ * <refsect2 id="GtkFontSelectionDialog-BUILDER-UI">
+ * <title>GtkFontSelectionDialog as GtkBuildable</title>
  * The GtkFontSelectionDialog implementation of the GtkBuildable interface
  * exposes the embedded #GtkFontSelection as internal child with the
- * name “font_selection”. It also exposes the buttons with the names
- * “ok_button”, “cancel_button” and “apply_button”.
+ * name "font_selection". It also exposes the buttons with the names
+ * "ok_button", "cancel_button" and "apply_button".
+ * </refsect2>
  */
 
 static void gtk_font_selection_dialog_buildable_interface_init     (GtkBuildableIface *iface);
@@ -1641,7 +1668,6 @@ static GObject * gtk_font_selection_dialog_buildable_get_internal_child (GtkBuil
 
 G_DEFINE_TYPE_WITH_CODE (GtkFontSelectionDialog, gtk_font_selection_dialog,
 			 GTK_TYPE_DIALOG,
-                         G_ADD_PRIVATE (GtkFontSelectionDialog)
 			 G_IMPLEMENT_INTERFACE (GTK_TYPE_BUILDABLE,
 						gtk_font_selection_dialog_buildable_interface_init))
 
@@ -1653,6 +1679,8 @@ gtk_font_selection_dialog_class_init (GtkFontSelectionDialogClass *klass)
   GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
 
   gtk_widget_class_set_accessible_role (widget_class, ATK_ROLE_FONT_CHOOSER);
+
+  g_type_class_add_private (klass, sizeof (GtkFontSelectionDialogPrivate));
 }
 
 static void
@@ -1662,7 +1690,9 @@ gtk_font_selection_dialog_init (GtkFontSelectionDialog *fontseldiag)
   GtkDialog *dialog = GTK_DIALOG (fontseldiag);
   GtkWidget *action_area, *content_area;
 
-  fontseldiag->priv = gtk_font_selection_dialog_get_instance_private (fontseldiag);
+  fontseldiag->priv = G_TYPE_INSTANCE_GET_PRIVATE (fontseldiag,
+                                                   GTK_TYPE_FONT_SELECTION_DIALOG,
+                                                   GtkFontSelectionDialogPrivate);
   priv = fontseldiag->priv;
 
   content_area = gtk_dialog_get_content_area (dialog);
@@ -1686,16 +1716,16 @@ gtk_font_selection_dialog_init (GtkFontSelectionDialog *fontseldiag)
 
   /* Create the action area */
   priv->cancel_button = gtk_dialog_add_button (dialog,
-                                               _("_Cancel"),
+                                               GTK_STOCK_CANCEL,
                                                GTK_RESPONSE_CANCEL);
 
   priv->apply_button = gtk_dialog_add_button (dialog,
-                                              _("_Apply"),
+                                              GTK_STOCK_APPLY,
                                               GTK_RESPONSE_APPLY);
   gtk_widget_hide (priv->apply_button);
 
   priv->ok_button = gtk_dialog_add_button (dialog,
-                                           _("_OK"),
+                                           GTK_STOCK_OK,
                                            GTK_RESPONSE_OK);
   gtk_widget_grab_default (priv->ok_button);
 
@@ -1717,7 +1747,7 @@ gtk_font_selection_dialog_init (GtkFontSelectionDialog *fontseldiag)
  *
  * Creates a new #GtkFontSelectionDialog.
  *
- * Returns: a new #GtkFontSelectionDialog
+ * Return value: a new #GtkFontSelectionDialog
  *
  * Deprecated: 3.2: Use #GtkFontChooserDialog
  */
@@ -1759,10 +1789,10 @@ gtk_font_selection_dialog_get_font_selection (GtkFontSelectionDialog *fsd)
  * gtk_font_selection_dialog_get_ok_button:
  * @fsd: a #GtkFontSelectionDialog
  *
- * Gets the “OK” button.
+ * Gets the 'OK' button.
  *
- * Returns: (transfer none): the #GtkWidget used in the dialog
- *     for the “OK” button.
+ * Return value: (transfer none): the #GtkWidget used in the dialog
+ *     for the 'OK' button.
  *
  * Since: 2.14
  *
@@ -1780,10 +1810,10 @@ gtk_font_selection_dialog_get_ok_button (GtkFontSelectionDialog *fsd)
  * gtk_font_selection_dialog_get_cancel_button:
  * @fsd: a #GtkFontSelectionDialog
  *
- * Gets the “Cancel” button.
+ * Gets the 'Cancel' button.
  *
- * Returns: (transfer none): the #GtkWidget used in the dialog
- *     for the “Cancel” button.
+ * Return value: (transfer none): the #GtkWidget used in the dialog
+ *     for the 'Cancel' button.
  *
  * Since: 2.14
  *
@@ -1834,11 +1864,11 @@ gtk_font_selection_dialog_buildable_get_internal_child (GtkBuildable *buildable,
  * Note that this can be a different string than what you set with 
  * gtk_font_selection_dialog_set_font_name(), as the font selection widget
  * may normalize font names and thus return a string with a different 
- * structure. For example, “Helvetica Italic Bold 12” could be normalized 
- * to “Helvetica Bold Italic 12”.  Use pango_font_description_equal()
+ * structure. For example, "Helvetica Italic Bold 12" could be normalized 
+ * to "Helvetica Bold Italic 12".  Use pango_font_description_equal()
  * if you want to compare two font descriptions.
  * 
- * Returns: A string with the name of the current font, or %NULL if no 
+ * Return value: A string with the name of the current font, or %NULL if no 
  *     font is selected. You must free this string with g_free().
  *
  * Deprecated: 3.2: Use #GtkFontChooserDialog
@@ -1858,11 +1888,11 @@ gtk_font_selection_dialog_get_font_name (GtkFontSelectionDialog *fsd)
 /**
  * gtk_font_selection_dialog_set_font_name:
  * @fsd: a #GtkFontSelectionDialog
- * @fontname: a font name like “Helvetica 12” or “Times Bold 18”
+ * @fontname: a font name like "Helvetica 12" or "Times Bold 18"
  *
  * Sets the currently selected font. 
  * 
- * Returns: %TRUE if the font selected in @fsd is now the
+ * Return value: %TRUE if the font selected in @fsd is now the
  *     @fontname specified, %FALSE otherwise. 
  *
  * Deprecated: 3.2: Use #GtkFontChooserDialog
@@ -1887,7 +1917,7 @@ gtk_font_selection_dialog_set_font_name (GtkFontSelectionDialog *fsd,
  *
  * Gets the text displayed in the preview area.
  * 
- * Returns: the text displayed in the preview area. 
+ * Return value: the text displayed in the preview area. 
  *     This string is owned by the widget and should not be 
  *     modified or freed 
  *
