@@ -59,10 +59,8 @@ struct _GtkApplicationClass
   void (*window_removed) (GtkApplication *application,
                           GtkWindow      *window);
 
-  void (*quit)           (GtkApplication *application);
-
   /*< private >*/
-  gpointer padding[11];
+  gpointer padding[12];
 };
 
 GType            gtk_application_get_type      (void) G_GNUC_CONST;
@@ -110,16 +108,6 @@ void             gtk_application_uninhibit          (GtkApplication
 gboolean         gtk_application_is_inhibited       (GtkApplication             *application,
                                                      GtkApplicationInhibitFlags  flags);
 
-typedef enum {
-  GTK_APPLICATION_LOGOUT,
-  GTK_APPLICATION_REBOOT,
-  GTK_APPLICATION_SHUTDOWN
-} GtkApplicationEndSessionStyle;
-
-gboolean         gtk_application_end_session        (GtkApplication                *application,
-                                                     GtkApplicationEndSessionStyle  style,
-                                                     gboolean                       request_confirmation);
-
 G_END_DECLS
 
 #endif /* __GTK_APPLICATION_H__ */
