@@ -12,13 +12,15 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library. If not, see <http://www.gnu.org/licenses/>.
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
 #include "config.h"
 
 #include "gtkdialog.h"
-#include "gtkbutton.h"
+#include "gtkstock.h"
 #include "gtkbox.h"
 #include "gtkprivate.h"
 #include "gtkintl.h"
@@ -58,7 +60,6 @@ enum
 static void gtk_color_chooser_dialog_iface_init (GtkColorChooserInterface *iface);
 
 G_DEFINE_TYPE_WITH_CODE (GtkColorChooserDialog, gtk_color_chooser_dialog, GTK_TYPE_DIALOG,
-                         G_ADD_PRIVATE (GtkColorChooserDialog)
                          G_IMPLEMENT_INTERFACE (GTK_TYPE_COLOR_CHOOSER,
                                                 gtk_color_chooser_dialog_iface_init))
 
@@ -70,52 +71,87 @@ propagate_notify (GObject               *o,
   g_object_notify (G_OBJECT (cc), pspec->name);
 }
 
-static void
-save_color (GtkColorChooserDialog *dialog)
-{
-  GdkRGBA color;
-
-  /* This causes the color chooser widget to save the
-   * selected and custom colors to GSettings.
-   */
-  gtk_color_chooser_get_rgba (GTK_COLOR_CHOOSER (dialog), &color);
-  gtk_color_chooser_set_rgba (GTK_COLOR_CHOOSER (dialog), &color);
-}
-
 static void
 color_activated_cb (GtkColorChooser *chooser,
                     GdkRGBA         *color,
                     GtkDialog       *dialog)
 {
-  save_color (GTK_COLOR_CHOOSER_DIALOG (dialog));
   gtk_dialog_response (dialog, GTK_RESPONSE_OK);
 }
 
-static void
-selected_cb (GtkButton *button,
-             GtkDialog *dialog)
-{
-  save_color (GTK_COLOR_CHOOSER_DIALOG (dialog));
-}
-
 static void
 gtk_color_chooser_dialog_init (GtkColorChooserDialog *cc)
 {
-  cc->priv = gtk_color_chooser_dialog_get_instance_private (cc);
+  GtkColorChooserDialogPrivate *priv;
+  GtkDialog *dialog = GTK_DIALOG (cc);
+  GtkWidget *action_area, *content_area;
 
-  gtk_widget_init_template (GTK_WIDGET (cc));
+  cc->priv = G_TYPE_INSTANCE_GET_PRIVATE (cc,
+                                          GTK_TYPE_COLOR_CHOOSER_DIALOG,
+                                          GtkColorChooserDialogPrivate);
+  priv = cc->priv;
+
+  content_area = gtk_dialog_get_content_area (dialog);
+  action_area = gtk_dialog_get_action_area (dialog);
+
+  gtk_container_set_border_width (GTK_CONTAINER (dialog), 5);
+  gtk_box_set_spacing (GTK_BOX (content_area), 2); /* 2 * 5 + 2 = 12 */
+  gtk_container_set_border_width (GTK_CONTAINER (action_area), 5);
+  gtk_box_set_spacing (GTK_BOX (action_area), 6);
+
+  gtk_widget_push_composite_child ();
+
+  gtk_window_set_resizable (GTK_WINDOW (cc), FALSE);
+
+  /* Create the content area */
+  priv->chooser = gtk_color_chooser_widget_new ();
+  gtk_container_set_border_width (GTK_CONTAINER (priv->chooser), 5);
+  gtk_widget_show (priv->chooser);
+  gtk_box_pack_start (GTK_BOX (content_area),
+                      priv->chooser, TRUE, TRUE, 0);
+
+  g_signal_connect (priv->chooser, "notify::rgba",
+                    G_CALLBACK (propagate_notify), cc);
+
+  g_signal_connect (priv->chooser, "notify::show-editor",
+                    G_CALLBACK (propagate_notify), cc);
+
+  g_signal_connect (priv->chooser, "color-activated",
+                    G_CALLBACK (color_activated_cb), cc);
+
+  /* Create the action area */
+  priv->cancel_button = gtk_dialog_add_button (dialog,
+                                               GTK_STOCK_CANCEL,
+                                               GTK_RESPONSE_CANCEL);
+  priv->select_button = gtk_dialog_add_button (dialog,
+                                               _("_Select"),
+                                               GTK_RESPONSE_OK);
+  gtk_widget_grab_default (priv->select_button);
+
+  gtk_dialog_set_alternative_button_order (dialog,
+                                           GTK_RESPONSE_OK,
+                                           GTK_RESPONSE_CANCEL,
+                                           -1);
+
+  gtk_window_set_title (GTK_WINDOW (cc), _("Select a Color"));
+
+  gtk_widget_pop_composite_child ();
 }
 
 static void
-gtk_color_chooser_dialog_map (GtkWidget *widget)
+gtk_color_chooser_dialog_response (GtkDialog *dialog,
+                                   gint       response_id)
 {
-  /* We never want the dialog to come up with the editor,
-   * even if it was showing the editor the last time it was used.
-   */
-  g_object_set (GTK_COLOR_CHOOSER_DIALOG (widget)->priv->chooser,
-                "show-editor", FALSE, NULL);
+  if (response_id == GTK_RESPONSE_OK)
+    {
+      GdkRGBA color;
 
-  GTK_WIDGET_CLASS (gtk_color_chooser_dialog_parent_class)->map (widget);
+      gtk_color_chooser_get_rgba (GTK_COLOR_CHOOSER (dialog), &color);
+      gtk_color_chooser_set_rgba (GTK_COLOR_CHOOSER (dialog), &color);
+    }
+
+  g_object_set (GTK_COLOR_CHOOSER_DIALOG (dialog)->priv->chooser,
+                "show-editor", FALSE, NULL);
 }
 
 static void
@@ -185,12 +221,12 @@ static void
 gtk_color_chooser_dialog_class_init (GtkColorChooserDialogClass *class)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (class);
-  GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (class);
+  GtkDialogClass *dialog_class = GTK_DIALOG_CLASS (class);
 
   object_class->get_property = gtk_color_chooser_dialog_get_property;
   object_class->set_property = gtk_color_chooser_dialog_set_property;
 
-  widget_class->map = gtk_color_chooser_dialog_map;
+  dialog_class->response = gtk_color_chooser_dialog_response;
 
   g_object_class_override_property (object_class, PROP_RGBA, "rgba");
   g_object_class_override_property (object_class, PROP_USE_ALPHA, "use-alpha");
@@ -198,16 +234,8 @@ gtk_color_chooser_dialog_class_init (GtkColorChooserDialogClass *class)
       g_param_spec_boolean ("show-editor", P_("Show editor"), P_("Show editor"),
                             FALSE, GTK_PARAM_READWRITE));
 
-  /* Bind class to template
-   */
-  gtk_widget_class_set_template_from_resource (widget_class,
-					       "/org/gtk/libgtk/gtkcolorchooserdialog.ui");
-  gtk_widget_class_bind_template_child_private (widget_class, GtkColorChooserDialog, chooser);
-  gtk_widget_class_bind_template_child_private (widget_class, GtkColorChooserDialog, cancel_button);
-  gtk_widget_class_bind_template_child_private (widget_class, GtkColorChooserDialog, select_button);
-  gtk_widget_class_bind_template_callback (widget_class, selected_cb);
-  gtk_widget_class_bind_template_callback (widget_class, propagate_notify);
-  gtk_widget_class_bind_template_callback (widget_class, color_activated_cb);
+
+  g_type_class_add_private (class, sizeof (GtkColorChooserDialogPrivate));
 }
 
 static void
@@ -230,7 +258,7 @@ gtk_color_chooser_dialog_set_rgba (GtkColorChooser *chooser,
 
 static void
 gtk_color_chooser_dialog_add_palette (GtkColorChooser *chooser,
-                                      GtkOrientation   orientation,
+                                      gboolean         horizontal,
                                       gint             colors_per_line,
                                       gint             n_colors,
                                       GdkRGBA         *colors)
@@ -238,7 +266,7 @@ gtk_color_chooser_dialog_add_palette (GtkColorChooser *chooser,
   GtkColorChooserDialog *cc = GTK_COLOR_CHOOSER_DIALOG (chooser);
 
   gtk_color_chooser_add_palette (GTK_COLOR_CHOOSER (cc->priv->chooser),
-                                 orientation, colors_per_line, n_colors, colors);
+                                 horizontal, colors_per_line, n_colors, colors);
 }
 
 static void
