@@ -13,7 +13,9 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library. If not, see <http://www.gnu.org/licenses/>.
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
 #include "config.h"
@@ -290,26 +292,10 @@ accessible_color_name (GdkRGBA *color)
                             scale_round (color->blue, 100));
 }
 
-static void
-remove_palette (GtkColorChooserWidget *cc)
-{
-  GList *children, *l;
-  GtkWidget *widget;
-
-  children = gtk_container_get_children (GTK_CONTAINER (cc->priv->palette));
-  for (l = children; l; l = l->next)
-    {
-      widget = l->data;
-      if (widget == cc->priv->custom_label || widget == cc->priv->custom)
-        continue;
-      gtk_container_remove (GTK_CONTAINER (cc->priv->palette), widget);
-    }
-  g_list_free (children);
-}
 
 static void
 add_palette (GtkColorChooserWidget  *cc,
-             GtkOrientation          orientation,
+             gboolean                horizontal,
              gint                    colors_per_line,
              gint                    n_colors,
              GdkRGBA                *colors,
@@ -322,12 +308,6 @@ add_palette (GtkColorChooserWidget  *cc,
   gint i;
   gint left, right;
 
-  if (colors == NULL)
-    {
-      remove_palette (cc);
-      return;
-    }
-
   grid = gtk_grid_new ();
   gtk_widget_set_margin_bottom (grid, 12);
   gtk_grid_set_row_spacing (GTK_GRID (grid), 2);
@@ -368,7 +348,7 @@ add_palette (GtkColorChooserWidget  *cc,
       line = i / colors_per_line;
       pos = i % colors_per_line;
 
-      if (orientation == GTK_ORIENTATION_HORIZONTAL)
+      if (horizontal)
         {
             if (pos == left)
               gtk_style_context_add_class (gtk_widget_get_style_context (p), GTK_STYLE_CLASS_LEFT);
@@ -394,10 +374,22 @@ add_palette (GtkColorChooserWidget  *cc,
 static void
 remove_default_palette (GtkColorChooserWidget *cc)
 {
+  GList *children, *l;
+  GtkWidget *widget;
+
   if (!cc->priv->has_default_palette)
     return;
 
-  remove_palette (cc);
+  children = gtk_container_get_children (GTK_CONTAINER (cc->priv->palette));
+  for (l = children; l; l = l->next)
+    {
+      widget = l->data;
+      if (widget == cc->priv->custom_label || widget == cc->priv->custom)
+        continue;
+      gtk_container_remove (GTK_CONTAINER (cc->priv->palette), widget);
+    }
+  g_list_free (children);
+
   cc->priv->has_default_palette = FALSE;
 }
 
@@ -473,12 +465,12 @@ add_default_palette (GtkColorChooserWidget *cc)
     for (j = 0; j < 3; j++)
       gdk_rgba_parse (&colors[i*3 + j], default_colors[i][j]);
 
-  add_palette (cc, GTK_ORIENTATION_VERTICAL, 3, 9*3, colors, color_names);
+  add_palette (cc, FALSE, 3, 9*3, colors, color_names);
 
   for (i = 0; i < 9; i++)
     gdk_rgba_parse (&colors[i], default_grays[i]);
 
-  add_palette (cc, GTK_ORIENTATION_HORIZONTAL, 9, 9, colors, gray_names);
+  add_palette (cc, TRUE, 9, 9, colors, gray_names);
 
   cc->priv->has_default_palette = TRUE;
 }
@@ -773,7 +765,7 @@ gtk_color_chooser_widget_set_rgba (GtkColorChooser *chooser,
 
 static void
 gtk_color_chooser_widget_add_palette (GtkColorChooser *chooser,
-                                      GtkOrientation   orientation,
+                                      gboolean         horizontal,
                                       gint             colors_per_line,
                                       gint             n_colors,
                                       GdkRGBA         *colors)
@@ -781,7 +773,7 @@ gtk_color_chooser_widget_add_palette (GtkColorChooser *chooser,
   GtkColorChooserWidget *cc = GTK_COLOR_CHOOSER_WIDGET (chooser);
 
   remove_default_palette (cc);
-  add_palette (cc, orientation, colors_per_line, n_colors, colors, NULL);
+  add_palette (cc, horizontal, colors_per_line, n_colors, colors, NULL);
 }
 
 static void
