@@ -12,7 +12,16 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library. If not, see <http://www.gnu.org/licenses/>.
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
+ */
+
+/* TODO
+ * - touch
+ * - accessible relations for popups
+ * - saving per-application (?)
+ * - better popup theming
  */
 
 #include "config.h"
@@ -24,18 +33,14 @@
 #include "gtkcolorscaleprivate.h"
 #include "gtkcolorswatchprivate.h"
 #include "gtkcolorutils.h"
-#include "gtkcolorpickerprivate.h"
 #include "gtkgrid.h"
-#include "gtkbutton.h"
-#include "gtkintl.h"
 #include "gtkorientable.h"
 #include "gtkentry.h"
 #include "gtkoverlay.h"
 #include "gtkadjustment.h"
 #include "gtklabel.h"
-#include "gtkrender.h"
 #include "gtkspinbutton.h"
-#include "gtkstylecontext.h"
+#include "gtkintl.h"
 
 #include <math.h>
 
@@ -63,11 +68,6 @@ struct _GtkColorEditorPrivate
   GtkAdjustment *v_adj;
   GtkAdjustment *a_adj;
 
-  GtkWidget *picker_button;
-  GtkColorPicker *picker;
-
-  gint popup_position;
-
   guint text_changed : 1;
   guint use_alpha    : 1;
 };
@@ -82,7 +82,6 @@ enum
 static void gtk_color_editor_iface_init (GtkColorChooserInterface *iface);
 
 G_DEFINE_TYPE_WITH_CODE (GtkColorEditor, gtk_color_editor, GTK_TYPE_BOX,
-                         G_ADD_PRIVATE (GtkColorEditor)
                          G_IMPLEMENT_INTERFACE (GTK_TYPE_COLOR_CHOOSER,
                                                 gtk_color_editor_iface_init))
 
@@ -177,12 +176,10 @@ dismiss_current_popup (GtkColorEditor *editor)
     {
       gtk_widget_hide (editor->priv->current_popup);
       editor->priv->current_popup = NULL;
-      editor->priv->popup_position = 0;
       if (editor->priv->popdown_focus)
         {
-          if (gtk_widget_is_visible (editor->priv->popdown_focus))
-            gtk_widget_grab_focus (editor->priv->popdown_focus);
-          g_clear_object (&editor->priv->popdown_focus);
+          gtk_widget_grab_focus (editor->priv->popdown_focus);
+          editor->priv->popdown_focus = NULL;
         }
     }
 }
@@ -194,39 +191,29 @@ popup_edit (GtkWidget      *widget,
   GtkWidget *popup = NULL;
   GtkWidget *toplevel;
   GtkWidget *focus;
-  gint position;
-  gint s, e;
 
   if (widget == editor->priv->sv_plane)
     {
       popup = editor->priv->sv_popup;
       focus = editor->priv->s_entry;
-      position = 0;
     }
   else if (widget == editor->priv->h_slider)
     {
       popup = editor->priv->h_popup;
       focus = editor->priv->h_entry;
-      gtk_range_get_slider_range (GTK_RANGE (editor->priv->h_slider), &s, &e);
-      position = (s + e) / 2;
     }
   else if (widget == editor->priv->a_slider)
     {
       popup = editor->priv->a_popup;
       focus = editor->priv->a_entry;
-      gtk_range_get_slider_range (GTK_RANGE (editor->priv->a_slider), &s, &e);
-      position = (s + e) / 2;
     }
 
-  if (popup == editor->priv->current_popup)
-    dismiss_current_popup (editor);
-  else if (popup)
+  if (popup)
     {
       dismiss_current_popup (editor);
       toplevel = gtk_widget_get_toplevel (GTK_WIDGET (editor));
-      g_set_object (&editor->priv->popdown_focus, gtk_window_get_focus (GTK_WINDOW (toplevel)));
+      editor->priv->popdown_focus = gtk_window_get_focus (GTK_WINDOW (toplevel));
       editor->priv->current_popup = popup;
-      editor->priv->popup_position = position;
       gtk_widget_show (popup);
       gtk_widget_grab_focus (focus);
     }
@@ -258,21 +245,16 @@ get_child_position (GtkOverlay     *overlay,
 
   gtk_widget_get_preferred_size (widget, &req, NULL);
 
-  allocation->x = 0;
-  allocation->y = 0;
   allocation->width = req.width;
   allocation->height = req.height;
 
   if (widget == editor->priv->sv_popup)
     {
-      gtk_widget_translate_coordinates (editor->priv->sv_plane,
-                                        gtk_widget_get_parent (editor->priv->grid),
-                                        0, -6,
-                                        &allocation->x, &allocation->y);
       if (gtk_widget_get_direction (GTK_WIDGET (overlay)) == GTK_TEXT_DIR_RTL)
         allocation->x = 0;
       else
         allocation->x = gtk_widget_get_allocated_width (GTK_WIDGET (overlay)) - req.width;
+      allocation->y = req.height / 3;
     }
   else if (widget == editor->priv->h_popup)
     {
@@ -282,12 +264,12 @@ get_child_position (GtkOverlay     *overlay,
       if (gtk_widget_get_direction (GTK_WIDGET (overlay)) == GTK_TEXT_DIR_RTL)
         gtk_widget_translate_coordinates (editor->priv->h_slider,
                                           gtk_widget_get_parent (editor->priv->grid),
-                                          - req.width - 6, editor->priv->popup_position - req.height / 2,
+                                          - req.width, (s + e - req.height) / 2,
                                           &allocation->x, &allocation->y);
       else
         gtk_widget_translate_coordinates (editor->priv->h_slider,
                                           gtk_widget_get_parent (editor->priv->grid),
-                                          alloc.width + 6, editor->priv->popup_position - req.height / 2,
+                                          alloc.width, (s + e - req.height) / 2,
                                           &allocation->x, &allocation->y);
     }
   else if (widget == editor->priv->a_popup)
@@ -297,7 +279,7 @@ get_child_position (GtkOverlay     *overlay,
 
       gtk_widget_translate_coordinates (editor->priv->a_slider,
                                         gtk_widget_get_parent (editor->priv->grid),
-                                        editor->priv->popup_position - req.width / 2, - req.height - 6,
+                                        (s + e - req.width) / 2, - req.height,
                                         &allocation->x, &allocation->y);
     }
   else
@@ -358,92 +340,180 @@ popup_draw (GtkWidget      *popup,
   return FALSE;
 }
 
-static void
-color_picked (GObject      *source,
-              GAsyncResult *res,
-              gpointer      data)
+static GtkWidget *
+create_popup (GtkColorEditor *editor,
+              GtkWidget      *attach,
+              GtkWidget      *contents)
 {
-  GtkColorPicker *picker = GTK_COLOR_PICKER (source);
-  GtkColorEditor *editor = data;
-  GError *error = NULL;
-  GdkRGBA *color;
+  GtkWidget *popup;
 
-  color = gtk_color_picker_pick_finish (picker, res, &error);
-  if (color == NULL)
-    {
-      g_warning ("Picking color failed: %s", error->message);
-      g_error_free (error);
-    }
-  else
-    {
-      gtk_color_chooser_set_rgba (GTK_COLOR_CHOOSER (editor), color);
-      gdk_rgba_free (color);
-    }
-}
+  g_object_set (contents, "margin", 12, NULL);
 
-static void
-pick_color (GtkButton      *button,
-            GtkColorEditor *editor)
-{
-  gtk_color_picker_pick (editor->priv->picker, color_picked, editor);
+  popup = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
+  gtk_style_context_add_class (gtk_widget_get_style_context (popup), GTK_STYLE_CLASS_TOOLTIP);
+  gtk_container_add (GTK_CONTAINER (popup), contents);
+
+  gtk_widget_show_all (contents);
+  gtk_widget_set_no_show_all (popup, TRUE);
+
+  g_signal_connect (popup, "draw", G_CALLBACK (popup_draw), editor);
+
+  gtk_overlay_add_overlay (GTK_OVERLAY (editor->priv->overlay), popup);
+  g_signal_connect (attach, "popup-menu", G_CALLBACK (popup_edit), editor);
+
+  return popup;
 }
 
 static void
 gtk_color_editor_init (GtkColorEditor *editor)
 {
-  editor->priv = gtk_color_editor_get_instance_private (editor);
+  GtkWidget *grid;
+  GtkWidget *slider;
+  GtkWidget *entry;
+  GtkWidget *swatch;
+  GtkAdjustment *h_adj, *s_adj, *v_adj, *a_adj;
+  AtkObject *atk_obj;
+  GdkRGBA transparent = { 0, 0, 0, 0 };
+
+  editor->priv = G_TYPE_INSTANCE_GET_PRIVATE (editor,
+                                              GTK_TYPE_COLOR_EDITOR,
+                                              GtkColorEditorPrivate);
   editor->priv->use_alpha = TRUE;
 
-  g_type_ensure (GTK_TYPE_COLOR_SCALE);
-  g_type_ensure (GTK_TYPE_COLOR_PLANE);
-  g_type_ensure (GTK_TYPE_COLOR_SWATCH);
-  gtk_widget_init_template (GTK_WIDGET (editor));
+  editor->priv->h_adj = h_adj = gtk_adjustment_new (0, 0, 1, 0.01, 0.1, 0);
+  editor->priv->s_adj = s_adj = gtk_adjustment_new (0, 0, 1, 0.01, 0.1, 0);
+  editor->priv->v_adj = v_adj = gtk_adjustment_new (0, 0, 1, 0.01, 0.1, 0);
+  editor->priv->a_adj = a_adj = gtk_adjustment_new (0, 0, 1, 0.01, 0.1, 0);
 
-  /* Some post processing is needed in code to set this up */
-  gtk_widget_set_events (editor->priv->swatch,
-			 gtk_widget_get_events (editor->priv->swatch)
+  g_object_ref_sink (h_adj);
+  g_object_ref_sink (s_adj);
+  g_object_ref_sink (v_adj);
+  g_object_ref_sink (a_adj);
+
+  g_signal_connect_swapped (h_adj, "value-changed", G_CALLBACK (hsv_changed), editor);
+  g_signal_connect_swapped (s_adj, "value-changed", G_CALLBACK (hsv_changed), editor);
+  g_signal_connect_swapped (v_adj, "value-changed", G_CALLBACK (hsv_changed), editor);
+  g_signal_connect_swapped (a_adj, "value-changed", G_CALLBACK (hsv_changed), editor);
+
+  gtk_widget_push_composite_child ();
+
+  /* Construct the main UI */
+  editor->priv->swatch = swatch = gtk_color_swatch_new ();
+  gtk_color_swatch_set_selectable (GTK_COLOR_SWATCH (editor->priv->swatch), FALSE);
+  gtk_widget_set_events (swatch, gtk_widget_get_events (swatch)
                                  & ~(GDK_BUTTON_PRESS_MASK
                                      | GDK_BUTTON_RELEASE_MASK
                                      | GDK_KEY_PRESS_MASK
                                      | GDK_KEY_RELEASE_MASK));
+  gtk_widget_set_can_focus (swatch, FALSE);
 
-  if (gtk_widget_get_direction (editor->priv->h_slider) == GTK_TEXT_DIR_RTL)
-    gtk_style_context_add_class (gtk_widget_get_style_context (editor->priv->h_slider),
-                                 "marks-before");
+  editor->priv->entry = entry = gtk_entry_new ();
+  atk_obj = gtk_widget_get_accessible (entry);
+  atk_object_set_role (atk_obj, ATK_ROLE_ENTRY);
+  atk_object_set_name (atk_obj, _("Color Name"));
+  g_signal_connect (entry, "activate", G_CALLBACK (entry_apply), editor);
+  g_signal_connect (entry, "notify::text", G_CALLBACK (entry_text_changed), editor);
+  g_signal_connect (entry, "focus-out-event", G_CALLBACK (entry_focus_out), editor);
+
+  editor->priv->h_slider = slider = gtk_color_scale_new (h_adj, GTK_COLOR_SCALE_HUE);
+  gtk_orientable_set_orientation (GTK_ORIENTABLE (slider), GTK_ORIENTATION_VERTICAL);
+  if (gtk_widget_get_direction (slider) == GTK_TEXT_DIR_RTL)
+    gtk_style_context_add_class (gtk_widget_get_style_context (slider),
+                                 GTK_STYLE_CLASS_SCALE_HAS_MARKS_ABOVE);
   else
-    gtk_style_context_add_class (gtk_widget_get_style_context (editor->priv->h_slider),
-                                 "marks-after");
+    gtk_style_context_add_class (gtk_widget_get_style_context (slider),
+                                 GTK_STYLE_CLASS_SCALE_HAS_MARKS_BELOW);
 
-  /* Create the scaled popup adjustments manually here because connecting user data is not
-   * supported by template GtkBuilder xml (it would be possible to set this up in the xml
-   * but require 4 separate callbacks and would be rather ugly).
+  editor->priv->sv_plane = gtk_color_plane_new (h_adj, s_adj, v_adj);
+  gtk_widget_set_size_request (editor->priv->sv_plane, 300, 300);
+
+  editor->priv->a_slider = slider = gtk_color_scale_new (a_adj, GTK_COLOR_SCALE_ALPHA);
+  gtk_orientable_set_orientation (GTK_ORIENTABLE (slider), GTK_ORIENTATION_HORIZONTAL);
+  gtk_style_context_add_class (gtk_widget_get_style_context (slider),
+                               GTK_STYLE_CLASS_SCALE_HAS_MARKS_ABOVE);
+
+  editor->priv->grid = grid = gtk_grid_new ();
+  gtk_grid_set_row_spacing (GTK_GRID (grid), 12);
+  gtk_grid_set_column_spacing (GTK_GRID (grid), 12);
+
+  gtk_grid_attach (GTK_GRID (grid), editor->priv->swatch,   1, 0, 1, 1);
+  gtk_grid_attach (GTK_GRID (grid), editor->priv->entry,    2, 0, 1, 1);
+  gtk_grid_attach (GTK_GRID (grid), editor->priv->h_slider, 0, 1, 1, 1);
+  gtk_grid_attach (GTK_GRID (grid), editor->priv->sv_plane, 1, 1, 2, 1);
+  gtk_grid_attach (GTK_GRID (grid), editor->priv->a_slider, 1, 2, 2, 1);
+
+  /* This extra margin is necessary so we have room to the sides
+   * to place the popups as desired
    */
-  gtk_spin_button_set_adjustment (GTK_SPIN_BUTTON (editor->priv->h_entry), scaled_adjustment (editor->priv->h_adj, 100));
-  gtk_spin_button_set_adjustment (GTK_SPIN_BUTTON (editor->priv->s_entry), scaled_adjustment (editor->priv->s_adj, 100));
-  gtk_spin_button_set_adjustment (GTK_SPIN_BUTTON (editor->priv->v_entry), scaled_adjustment (editor->priv->v_adj, 100));
-  gtk_spin_button_set_adjustment (GTK_SPIN_BUTTON (editor->priv->a_entry), scaled_adjustment (editor->priv->a_adj, 100));
+  gtk_widget_set_margin_left (grid, 30);
+  gtk_widget_set_margin_right (grid, 30);
 
-  /* This can be setup in the .ui file, but requires work in Glade otherwise it cannot be edited there */
-  gtk_overlay_add_overlay (GTK_OVERLAY (editor->priv->overlay), editor->priv->sv_popup);
-  gtk_overlay_add_overlay (GTK_OVERLAY (editor->priv->overlay), editor->priv->h_popup);
-  gtk_overlay_add_overlay (GTK_OVERLAY (editor->priv->overlay), editor->priv->a_popup);
+  editor->priv->overlay = gtk_overlay_new ();
+  gtk_widget_override_background_color (editor->priv->overlay, 0, &transparent);
+  gtk_container_add (GTK_CONTAINER (editor->priv->overlay), grid);
 
-  gtk_style_context_remove_class (gtk_widget_get_style_context (editor->priv->swatch), "activatable");
+  /* Construct the sv popup */
+  editor->priv->s_entry = entry = gtk_spin_button_new (scaled_adjustment (s_adj, 100), 1, 0);
+  atk_obj = gtk_widget_get_accessible (entry);
+  atk_object_set_name (atk_obj, C_("Color channel", "Saturation"));
+  atk_object_set_role (atk_obj, ATK_ROLE_ENTRY);
+  g_signal_connect (entry, "key-press-event", G_CALLBACK (popup_key_press), editor);
 
-  editor->priv->picker = gtk_color_picker_new ();
-  if (editor->priv->picker == NULL)
-    gtk_widget_hide (editor->priv->picker_button);
-}
+  editor->priv->v_entry = entry = gtk_spin_button_new (scaled_adjustment (v_adj, 100), 1, 0);
+  atk_obj = gtk_widget_get_accessible (entry);
+  atk_object_set_name (atk_obj, C_("Color channel", "Value"));
+  atk_object_set_role (atk_obj, ATK_ROLE_ENTRY);
+  g_signal_connect (entry, "key-press-event", G_CALLBACK (popup_key_press), editor);
 
-static void
-gtk_color_editor_dispose (GObject *object)
-{
-  GtkColorEditor *editor = GTK_COLOR_EDITOR (object);
+  grid = gtk_grid_new ();
+  gtk_grid_set_row_spacing (GTK_GRID (grid), 6);
+  gtk_grid_set_column_spacing (GTK_GRID (grid), 6);
 
-  dismiss_current_popup (editor);
-  g_clear_object (&editor->priv->picker);
+  gtk_grid_attach (GTK_GRID (grid), gtk_label_new (C_("Color channel", "S")), 0, 0, 1, 1);
+  gtk_grid_attach (GTK_GRID (grid), editor->priv->s_entry, 1, 0, 1, 1);
+  gtk_grid_attach (GTK_GRID (grid), gtk_label_new (C_("Color channel", "V")), 0, 1, 1, 1);
+  gtk_grid_attach (GTK_GRID (grid), editor->priv->v_entry, 1, 1, 1, 1);
 
-  G_OBJECT_CLASS (gtk_color_editor_parent_class)->dispose (object);
+  editor->priv->sv_popup = create_popup (editor, editor->priv->sv_plane, grid);
+
+  /* Construct the h popup */
+  editor->priv->h_entry = entry = gtk_spin_button_new (scaled_adjustment (h_adj, 100), 1, 0);
+  atk_obj = gtk_widget_get_accessible (entry);
+  atk_object_set_name (atk_obj, C_("Color channel", "Hue"));
+  atk_object_set_role (atk_obj, ATK_ROLE_ENTRY);
+  g_signal_connect (entry, "key-press-event", G_CALLBACK (popup_key_press), editor);
+
+  grid = gtk_grid_new ();
+  gtk_grid_set_column_spacing (GTK_GRID (grid), 6);
+
+  gtk_grid_attach (GTK_GRID (grid), gtk_label_new (C_("Color channel", "H")), 0, 0, 1, 1);
+  gtk_grid_attach (GTK_GRID (grid), editor->priv->h_entry, 1, 0, 1, 1);
+
+  editor->priv->h_popup = create_popup (editor, editor->priv->h_slider, grid);
+
+  /* Construct the a popup */
+  editor->priv->a_entry = entry = gtk_spin_button_new (scaled_adjustment (a_adj, 100), 1, 0);
+  atk_obj = gtk_widget_get_accessible (entry);
+  atk_object_set_name (atk_obj, C_("Color channel", "Alpha"));
+  atk_object_set_role (atk_obj, ATK_ROLE_ENTRY);
+  g_signal_connect (entry, "key-press-event", G_CALLBACK (popup_key_press), editor);
+
+  grid = gtk_grid_new ();
+  gtk_grid_set_column_spacing (GTK_GRID (grid), 6);
+
+  gtk_grid_attach (GTK_GRID (grid), gtk_label_new (C_("Color channel", "A")), 0, 0, 1, 1);
+  gtk_grid_attach (GTK_GRID (grid), editor->priv->a_entry, 1, 0, 1, 1);
+
+  editor->priv->a_popup = create_popup (editor, editor->priv->a_slider, grid);
+
+  /* Hook up popup positioning */
+  g_signal_connect (editor->priv->overlay, "get-child-position", G_CALLBACK (get_child_position), editor);
+  g_signal_connect (editor, "notify::visible", G_CALLBACK (dismiss_current_popup), NULL);
+
+  gtk_widget_show_all (editor->priv->overlay);
+  gtk_container_add (GTK_CONTAINER (editor), editor->priv->overlay);
+
+  gtk_widget_pop_composite_child ();
 }
 
 static void
@@ -508,54 +578,32 @@ gtk_color_editor_set_property (GObject      *object,
     }
 }
 
+static void
+gtk_color_editor_finalize (GObject *object)
+{
+  GtkColorEditor *editor = GTK_COLOR_EDITOR (object);
+
+  g_clear_object (&editor->priv->h_adj);
+  g_clear_object (&editor->priv->s_adj);
+  g_clear_object (&editor->priv->v_adj);
+  g_clear_object (&editor->priv->a_adj);
+
+  G_OBJECT_CLASS (gtk_color_editor_parent_class)->finalize (object);
+}
+
 static void
 gtk_color_editor_class_init (GtkColorEditorClass *class)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (class);
-  GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (class);
 
-  object_class->dispose = gtk_color_editor_dispose;
+  object_class->finalize = gtk_color_editor_finalize;
   object_class->get_property = gtk_color_editor_get_property;
   object_class->set_property = gtk_color_editor_set_property;
 
   g_object_class_override_property (object_class, PROP_RGBA, "rgba");
   g_object_class_override_property (object_class, PROP_USE_ALPHA, "use-alpha");
 
-  /* Bind class to template
-   */
-  gtk_widget_class_set_template_from_resource (widget_class,
-					       "/org/gtk/libgtk/ui/gtkcoloreditor.ui");
-
-  gtk_widget_class_bind_template_child_private (widget_class, GtkColorEditor, overlay);
-  gtk_widget_class_bind_template_child_private (widget_class, GtkColorEditor, grid);
-  gtk_widget_class_bind_template_child_private (widget_class, GtkColorEditor, swatch);
-  gtk_widget_class_bind_template_child_private (widget_class, GtkColorEditor, entry);
-  gtk_widget_class_bind_template_child_private (widget_class, GtkColorEditor, h_slider);
-  gtk_widget_class_bind_template_child_private (widget_class, GtkColorEditor, h_popup);
-  gtk_widget_class_bind_template_child_private (widget_class, GtkColorEditor, h_entry);
-  gtk_widget_class_bind_template_child_private (widget_class, GtkColorEditor, a_slider);
-  gtk_widget_class_bind_template_child_private (widget_class, GtkColorEditor, a_popup);
-  gtk_widget_class_bind_template_child_private (widget_class, GtkColorEditor, a_entry);
-  gtk_widget_class_bind_template_child_private (widget_class, GtkColorEditor, sv_plane);
-  gtk_widget_class_bind_template_child_private (widget_class, GtkColorEditor, sv_popup);
-  gtk_widget_class_bind_template_child_private (widget_class, GtkColorEditor, s_entry);
-  gtk_widget_class_bind_template_child_private (widget_class, GtkColorEditor, v_entry);
-  gtk_widget_class_bind_template_child_private (widget_class, GtkColorEditor, h_adj);
-  gtk_widget_class_bind_template_child_private (widget_class, GtkColorEditor, s_adj);
-  gtk_widget_class_bind_template_child_private (widget_class, GtkColorEditor, v_adj);
-  gtk_widget_class_bind_template_child_private (widget_class, GtkColorEditor, a_adj);
-  gtk_widget_class_bind_template_child_private (widget_class, GtkColorEditor, picker_button);
-
-  gtk_widget_class_bind_template_callback (widget_class, hsv_changed);
-  gtk_widget_class_bind_template_callback (widget_class, popup_draw);
-  gtk_widget_class_bind_template_callback (widget_class, popup_key_press);
-  gtk_widget_class_bind_template_callback (widget_class, dismiss_current_popup);
-  gtk_widget_class_bind_template_callback (widget_class, get_child_position);
-  gtk_widget_class_bind_template_callback (widget_class, entry_text_changed);
-  gtk_widget_class_bind_template_callback (widget_class, entry_apply);
-  gtk_widget_class_bind_template_callback (widget_class, entry_focus_out);
-  gtk_widget_class_bind_template_callback (widget_class, popup_edit);
-  gtk_widget_class_bind_template_callback (widget_class, pick_color);
+  g_type_class_add_private (class, sizeof (GtkColorEditorPrivate));
 }
 
 static void
