@@ -12,18 +12,17 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library. If not, see <http://www.gnu.org/licenses/>.
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
 #include "config.h"
 
 #include "gtkcolorplaneprivate.h"
 
-#include "gtkgesturedrag.h"
-#include "gtkgesturelongpress.h"
-#include "gtkaccessible.h"
-#include "gtkadjustment.h"
 #include "gtkcolorutils.h"
+#include "gtkaccessible.h"
 #include "gtkintl.h"
 
 struct _GtkColorPlanePrivate
@@ -33,19 +32,10 @@ struct _GtkColorPlanePrivate
   GtkAdjustment *v_adj;
 
   cairo_surface_t *surface;
-
-  GtkGesture *drag_gesture;
-  GtkGesture *long_press_gesture;
+  gboolean in_drag;
 };
 
-enum {
-  PROP_0,
-  PROP_H_ADJUSTMENT,
-  PROP_S_ADJUSTMENT,
-  PROP_V_ADJUSTMENT
-};
-
-G_DEFINE_TYPE_WITH_PRIVATE (GtkColorPlane, gtk_color_plane, GTK_TYPE_DRAWING_AREA)
+G_DEFINE_TYPE (GtkColorPlane, gtk_color_plane, GTK_TYPE_DRAWING_AREA)
 
 static void
 sv_to_xy (GtkColorPlane *plane,
@@ -172,44 +162,41 @@ create_surface (GtkColorPlane *plane)
   g_free (data);
 }
 
-static void
-plane_size_allocate (GtkWidget     *widget,
-                     GtkAllocation *allocation)
+static gboolean
+plane_configure (GtkWidget         *widget,
+                 GdkEventConfigure *event)
 {
-  GTK_WIDGET_CLASS (gtk_color_plane_parent_class)->size_allocate (widget, allocation);
-
   create_surface (GTK_COLOR_PLANE (widget));
+  return TRUE;
 }
 
 static void
-plane_realize (GtkWidget *widget)
+set_cross_grab (GtkWidget *widget,
+                GdkDevice *device,
+                guint32    time)
 {
-  GTK_WIDGET_CLASS (gtk_color_plane_parent_class)->realize (widget);
+  GdkCursor *cursor;
 
-  create_surface (GTK_COLOR_PLANE (widget));
+  cursor = gdk_cursor_new_for_display (gtk_widget_get_display (GTK_WIDGET (widget)),
+                                       GDK_CROSSHAIR);
+  gdk_device_grab (device,
+                   gtk_widget_get_window (widget),
+                   GDK_OWNERSHIP_NONE,
+                   FALSE,
+                   GDK_POINTER_MOTION_MASK
+                    | GDK_POINTER_MOTION_HINT_MASK
+                    | GDK_BUTTON_RELEASE_MASK,
+                   cursor,
+                   time);
+  g_object_unref (cursor);
 }
 
-static void
-set_cross_cursor (GtkWidget *widget,
-                  gboolean   enabled)
+static gboolean
+plane_grab_broken (GtkWidget          *widget,
+                   GdkEventGrabBroken *event)
 {
-  GdkCursor *cursor = NULL;
-  GdkWindow *window;
-  GdkDevice *device;
-
-  window = gtk_widget_get_window (widget);
-  device = gtk_gesture_get_device (GTK_COLOR_PLANE (widget)->priv->drag_gesture);
-
-  if (!window || !device)
-    return;
-
-  if (enabled)
-    cursor = gdk_cursor_new_from_name (gtk_widget_get_display (GTK_WIDGET (widget)), "crosshair");
-
-  gdk_window_set_device_cursor (window, device, cursor);
-
-  if (cursor)
-    g_object_unref (cursor);
+  GTK_COLOR_PLANE (widget)->priv->in_drag = FALSE;
+  return TRUE;
 }
 
 static void
@@ -241,15 +228,62 @@ update_color (GtkColorPlane *plane,
   gtk_widget_queue_draw (widget);
 }
 
-static void
-hold_action (GtkGestureLongPress *gesture,
-             gdouble              x,
-             gdouble              y,
-             GtkColorPlane       *plane)
+static gboolean
+plane_button_press (GtkWidget      *widget,
+                    GdkEventButton *event)
 {
-  gboolean handled;
+  GtkColorPlane *plane = GTK_COLOR_PLANE (widget);
 
-  g_signal_emit_by_name (plane, "popup-menu", &handled);
+  if (event->button == GDK_BUTTON_SECONDARY)
+    {
+      gboolean handled;
+
+      g_signal_emit_by_name (widget, "popup-menu", &handled);
+
+      return TRUE;
+    }
+
+  if (plane->priv->in_drag || event->button != GDK_BUTTON_PRIMARY)
+    return FALSE;
+
+  plane->priv->in_drag = TRUE;
+  set_cross_grab (widget, gdk_event_get_device ((GdkEvent*)event), event->time);
+  update_color (plane, event->x, event->y);
+  gtk_widget_grab_focus (widget);
+
+  return TRUE;
+}
+
+static gboolean
+plane_button_release (GtkWidget      *widget,
+                      GdkEventButton *event)
+{
+  GtkColorPlane *plane = GTK_COLOR_PLANE (widget);
+
+  if (!plane->priv->in_drag || event->button != GDK_BUTTON_PRIMARY)
+    return FALSE;
+
+  plane->priv->in_drag = FALSE;
+
+  update_color (plane, event->x, event->y);
+  gdk_device_ungrab (gdk_event_get_device ((GdkEvent *) event), event->time);
+
+  return TRUE;
+}
+
+static gboolean
+plane_motion_notify (GtkWidget      *widget,
+                     GdkEventMotion *event)
+{
+  GtkColorPlane *plane = GTK_COLOR_PLANE (widget);
+
+  if (!plane->priv->in_drag)
+    return FALSE;
+
+  gdk_event_request_motions (event);
+  update_color (plane, event->x, event->y);
+
+  return TRUE;
 }
 
 static void
@@ -338,67 +372,17 @@ plane_key_press (GtkWidget   *widget,
   return TRUE;
 }
 
-static void
-plane_drag_gesture_begin (GtkGestureDrag *gesture,
-                          gdouble         start_x,
-                          gdouble         start_y,
-                          GtkColorPlane  *plane)
-{
-  guint button;
-
-  button = gtk_gesture_single_get_current_button (GTK_GESTURE_SINGLE (gesture));
-
-  if (button == GDK_BUTTON_SECONDARY)
-    {
-      gboolean handled;
-
-      g_signal_emit_by_name (plane, "popup-menu", &handled);
-    }
-
-  if (button != GDK_BUTTON_PRIMARY)
-    {
-      gtk_gesture_set_state (GTK_GESTURE (gesture), GTK_EVENT_SEQUENCE_DENIED);
-      return;
-    }
-
-  set_cross_cursor (GTK_WIDGET (plane), TRUE);
-  update_color (plane, start_x, start_y);
-  gtk_widget_grab_focus (GTK_WIDGET (plane));
-  gtk_gesture_set_state (GTK_GESTURE (gesture), GTK_EVENT_SEQUENCE_CLAIMED);
-}
-
-static void
-plane_drag_gesture_update (GtkGestureDrag *gesture,
-                           gdouble         offset_x,
-                           gdouble         offset_y,
-                           GtkColorPlane  *plane)
-{
-  gdouble start_x, start_y;
-
-  gtk_gesture_drag_get_start_point (GTK_GESTURE_DRAG (gesture),
-                                    &start_x, &start_y);
-  update_color (plane, start_x + offset_x, start_y + offset_y);
-}
-
-static void
-plane_drag_gesture_end (GtkGestureDrag *gesture,
-                        gdouble         offset_x,
-                        gdouble         offset_y,
-                        GtkColorPlane  *plane)
-{
-  set_cross_cursor (GTK_WIDGET (plane), FALSE);
-}
-
 static void
 gtk_color_plane_init (GtkColorPlane *plane)
 {
   AtkObject *atk_obj;
 
-  plane->priv = gtk_color_plane_get_instance_private (plane);
+  plane->priv = G_TYPE_INSTANCE_GET_PRIVATE (plane,
+                                             GTK_TYPE_COLOR_PLANE,
+                                             GtkColorPlanePrivate);
 
   gtk_widget_set_can_focus (GTK_WIDGET (plane), TRUE);
   gtk_widget_set_events (GTK_WIDGET (plane), GDK_KEY_PRESS_MASK
-                                             | GDK_TOUCH_MASK
                                              | GDK_BUTTON_PRESS_MASK
                                              | GDK_BUTTON_RELEASE_MASK
                                              | GDK_POINTER_MOTION_MASK);
@@ -409,21 +393,6 @@ gtk_color_plane_init (GtkColorPlane *plane)
       atk_object_set_name (atk_obj, _("Color Plane"));
       atk_object_set_role (atk_obj, ATK_ROLE_COLOR_CHOOSER);
     }
-
-  plane->priv->drag_gesture = gtk_gesture_drag_new (GTK_WIDGET (plane));
-  g_signal_connect (plane->priv->drag_gesture, "drag-begin",
-		    G_CALLBACK (plane_drag_gesture_begin), plane);
-  g_signal_connect (plane->priv->drag_gesture, "drag-update",
-		    G_CALLBACK (plane_drag_gesture_update), plane);
-  g_signal_connect (plane->priv->drag_gesture, "drag-end",
-		    G_CALLBACK (plane_drag_gesture_end), plane);
-  gtk_gesture_single_set_button (GTK_GESTURE_SINGLE (plane->priv->drag_gesture), 0);
-
-  plane->priv->long_press_gesture = gtk_gesture_long_press_new (GTK_WIDGET (plane));
-  g_signal_connect (plane->priv->long_press_gesture, "pressed",
-                    G_CALLBACK (hold_action), plane);
-  gtk_gesture_single_set_touch_only (GTK_GESTURE_SINGLE (plane->priv->long_press_gesture),
-                                     TRUE);
 }
 
 static void
@@ -438,56 +407,9 @@ plane_finalize (GObject *object)
   g_clear_object (&plane->priv->s_adj);
   g_clear_object (&plane->priv->v_adj);
 
-  g_clear_object (&plane->priv->drag_gesture);
-  g_clear_object (&plane->priv->long_press_gesture);
-
   G_OBJECT_CLASS (gtk_color_plane_parent_class)->finalize (object);
 }
 
-static void
-plane_set_property (GObject      *object,
-		    guint         prop_id,
-		    const GValue *value,
-		    GParamSpec   *pspec)
-{
-  GtkColorPlane *plane = GTK_COLOR_PLANE (object);
-  GtkAdjustment *adjustment;
-
-  /* Construct only properties can only be set once, these are created
-   * only in order to be properly buildable from gtkcoloreditor.ui
-   */
-  switch (prop_id)
-    {
-    case PROP_H_ADJUSTMENT:
-      adjustment = g_value_get_object (value);
-      if (adjustment)
-	{
-	  plane->priv->h_adj = GTK_ADJUSTMENT (g_object_ref_sink (adjustment));
-	  g_signal_connect_swapped (adjustment, "value-changed", G_CALLBACK (h_changed), plane);
-	}
-      break;
-    case PROP_S_ADJUSTMENT:
-      adjustment = g_value_get_object (value);
-      if (adjustment)
-	{
-	  plane->priv->s_adj = GTK_ADJUSTMENT (g_object_ref_sink (adjustment));
-	  g_signal_connect_swapped (adjustment, "value-changed", G_CALLBACK (sv_changed), plane);
-	}
-      break;
-    case PROP_V_ADJUSTMENT:
-      adjustment = g_value_get_object (value);
-      if (adjustment)
-	{
-	  plane->priv->v_adj = GTK_ADJUSTMENT (g_object_ref_sink (adjustment));
-	  g_signal_connect_swapped (adjustment, "value-changed", G_CALLBACK (sv_changed), plane);
-	}
-      break;
-    default:
-      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-      break;
-    }
-}
-
 static void
 gtk_color_plane_class_init (GtkColorPlaneClass *class)
 {
@@ -495,39 +417,16 @@ gtk_color_plane_class_init (GtkColorPlaneClass *class)
   GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (class);
 
   object_class->finalize = plane_finalize;
-  object_class->set_property = plane_set_property;
 
   widget_class->draw = plane_draw;
-  widget_class->size_allocate = plane_size_allocate;
-  widget_class->realize = plane_realize;
+  widget_class->configure_event = plane_configure;
+  widget_class->button_press_event = plane_button_press;
+  widget_class->button_release_event = plane_button_release;
+  widget_class->motion_notify_event = plane_motion_notify;
+  widget_class->grab_broken_event = plane_grab_broken;
   widget_class->key_press_event = plane_key_press;
 
-  g_object_class_install_property (object_class,
-                                   PROP_H_ADJUSTMENT,
-                                   g_param_spec_object ("h-adjustment",
-                                                        "Hue Adjustment",
-                                                        "Hue Adjustment",
-							GTK_TYPE_ADJUSTMENT,
-							G_PARAM_WRITABLE |
-							G_PARAM_CONSTRUCT_ONLY));
-
-  g_object_class_install_property (object_class,
-                                   PROP_S_ADJUSTMENT,
-                                   g_param_spec_object ("s-adjustment",
-                                                        "Saturation Adjustment",
-                                                        "Saturation Adjustment",
-							GTK_TYPE_ADJUSTMENT,
-							G_PARAM_WRITABLE |
-							G_PARAM_CONSTRUCT_ONLY));
-
-  g_object_class_install_property (object_class,
-                                   PROP_V_ADJUSTMENT,
-                                   g_param_spec_object ("v-adjustment",
-                                                        "Value Adjustment",
-                                                        "Value Adjustment",
-							GTK_TYPE_ADJUSTMENT,
-							G_PARAM_WRITABLE |
-							G_PARAM_CONSTRUCT_ONLY));
+  g_type_class_add_private (class, sizeof (GtkColorPlanePrivate));
 }
 
 GtkWidget *
@@ -535,9 +434,16 @@ gtk_color_plane_new (GtkAdjustment *h_adj,
                      GtkAdjustment *s_adj,
                      GtkAdjustment *v_adj)
 {
-  return g_object_new (GTK_TYPE_COLOR_PLANE,
-                       "h-adjustment", h_adj,
-                       "s-adjustment", s_adj,
-                       "v-adjustment", v_adj,
-                       NULL);
+  GtkColorPlane *plane;
+
+  plane = (GtkColorPlane *) g_object_new (GTK_TYPE_COLOR_PLANE, NULL);
+
+  plane->priv->h_adj = g_object_ref_sink (h_adj);
+  plane->priv->s_adj = g_object_ref_sink (s_adj);
+  plane->priv->v_adj = g_object_ref_sink (v_adj);
+  g_signal_connect_swapped (h_adj, "value-changed", G_CALLBACK (h_changed), plane);
+  g_signal_connect_swapped (s_adj, "value-changed", G_CALLBACK (sv_changed), plane);
+  g_signal_connect_swapped (v_adj, "value-changed", G_CALLBACK (sv_changed), plane);
+
+  return (GtkWidget *)plane;
 }
