@@ -12,7 +12,9 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library. If not, see <http://www.gnu.org/licenses/>.
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
 #include "config.h"
@@ -20,6 +22,8 @@
 #include "gtkcolorswatchprivate.h"
 
 #include "gtkcolorchooserprivate.h"
+#include "gtkroundedboxprivate.h"
+#include "gtkthemingbackgroundprivate.h"
 #include "gtkdnd.h"
 #include "gtkicontheme.h"
 #include "gtkmain.h"
@@ -28,51 +32,27 @@
 #include "gtkmenushell.h"
 #include "gtkprivate.h"
 #include "gtkintl.h"
-#include "gtkrenderprivate.h"
-#include "gtkiconhelperprivate.h"
-#include "gtkcssnodeprivate.h"
-#include "gtkcsscustomgadgetprivate.h"
-#include "gtkwidgetprivate.h"
-#include "gtkstylecontextprivate.h"
-#include "a11y/gtkcolorswatchaccessibleprivate.h"
+#include "a11y/gtkcolorswatchaccessible.h"
 
 
-/*
- * GtkColorSwatch has two CSS nodes, the main one named colorswatch
- * and a subnode named overlay. The main node gets the .light or .dark
- * style classes added depending on the brightness of the color that
- * the swatch is showing.
- *
- * The color swatch has the .activatable style class by default. It can
- * be removed for non-activatable swatches.
- */
-
 struct _GtkColorSwatchPrivate
 {
   GdkRGBA color;
   gdouble radius[4];
   gchar *icon;
   guint    has_color        : 1;
+  guint    contains_pointer : 1;
   guint    use_alpha        : 1;
   guint    selectable       : 1;
-  guint    has_menu         : 1;
 
   GdkWindow *event_window;
-
-  GtkGesture *long_press_gesture;
-  GtkGesture *multipress_gesture;
-  GtkCssGadget *gadget;
-  GtkCssGadget *overlay_gadget;
-
-  GtkWidget *popover;
 };
 
 enum
 {
   PROP_ZERO,
   PROP_RGBA,
-  PROP_SELECTABLE,
-  PROP_HAS_MENU
+  PROP_SELECTABLE
 };
 
 enum
@@ -84,59 +64,62 @@ enum
 
 static guint signals[LAST_SIGNAL];
 
+G_DEFINE_TYPE (GtkColorSwatch, gtk_color_swatch, GTK_TYPE_WIDGET)
 
-G_DEFINE_TYPE_WITH_PRIVATE (GtkColorSwatch, gtk_color_swatch, GTK_TYPE_WIDGET)
+static void
+gtk_color_swatch_init (GtkColorSwatch *swatch)
+{
+  swatch->priv = G_TYPE_INSTANCE_GET_PRIVATE (swatch,
+                                              GTK_TYPE_COLOR_SWATCH,
+                                              GtkColorSwatchPrivate);
+
+  gtk_widget_set_can_focus (GTK_WIDGET (swatch), TRUE);
+  gtk_widget_set_has_window (GTK_WIDGET (swatch), FALSE);
+
+  swatch->priv->use_alpha = TRUE;
+  swatch->priv->selectable = TRUE;
+}
+
+#define INTENSITY(r, g, b) ((r) * 0.30 + (g) * 0.59 + (b) * 0.11)
+#define ACTIVE_BADGE_RADIUS 10
 
 static gboolean
 swatch_draw (GtkWidget *widget,
              cairo_t   *cr)
 {
-  gtk_css_gadget_draw (GTK_COLOR_SWATCH (widget)->priv->gadget, cr);
-
-  return FALSE;
-}
-
-#define INTENSITY(r, g, b) ((r) * 0.30 + (g) * 0.59 + (b) * 0.11)
-
-static gboolean
-gtk_color_swatch_render (GtkCssGadget *gadget,
-                         cairo_t      *cr,
-                         int           x,
-                         int           y,
-                         int           width,
-                         int           height,
-                         gpointer      data)
-{
-  GtkWidget *widget;
-  GtkColorSwatch *swatch;
+  GtkColorSwatch *swatch = (GtkColorSwatch*)widget;
+  GtkThemingBackground background;
+  gdouble width, height;
   GtkStyleContext *context;
+  GtkStateFlags state;
+  GtkIconTheme *theme;
+  GtkIconInfo *icon_info = NULL;
 
-  widget = gtk_css_gadget_get_owner (gadget);
-  swatch = GTK_COLOR_SWATCH (widget);
+  theme = gtk_icon_theme_get_default ();
   context = gtk_widget_get_style_context (widget);
+  state = gtk_widget_get_state_flags (widget);
+  width = gtk_widget_get_allocated_width (widget);
+  height = gtk_widget_get_allocated_height (widget);
+
+  cairo_save (cr);
+
+  gtk_style_context_save (context);
+  gtk_style_context_set_state (context, state);
+
+  _gtk_theming_background_init_from_context (&background, context,
+                                             0, 0, width, height,
+                                             GTK_JUNCTION_NONE);
 
   if (swatch->priv->has_color)
     {
       cairo_pattern_t *pattern;
       cairo_matrix_t matrix;
-      GtkAllocation allocation, border_allocation;
-
-      gtk_widget_get_allocation (widget, &allocation);
-      gtk_css_gadget_get_border_allocation (gadget, &border_allocation, NULL);
-
-      border_allocation.x -= allocation.x;
-      border_allocation.y -= allocation.y;
-
-      gtk_render_content_path (context, cr,
-                               border_allocation.x,
-                               border_allocation.y,
-                               border_allocation.width,
-                               border_allocation.height);
 
       if (swatch->priv->use_alpha)
         {
           cairo_save (cr);
 
+          _gtk_rounded_box_path (&background.clip_box, cr);
           cairo_clip_preserve (cr);
 
           cairo_set_source_rgb (cr, 0.33, 0.33, 0.33);
@@ -152,22 +135,105 @@ gtk_color_swatch_render (GtkCssGadget *gadget,
 
           cairo_restore (cr);
 
-          gdk_cairo_set_source_rgba (cr, &swatch->priv->color);
+          background.bg_color = swatch->priv->color;
         }
       else
         {
-          cairo_set_source_rgb (cr,
-                                swatch->priv->color.red,
-                                swatch->priv->color.green,
-                                swatch->priv->color.blue);
+          background.bg_color = swatch->priv->color;
+          background.bg_color.alpha = 1.0;
         }
 
-      cairo_fill (cr);
+      _gtk_theming_background_render (&background, cr);
+    }
+  else
+    _gtk_theming_background_render (&background, cr);
+
+  gtk_render_frame (context, cr,
+                    0, 0, width, height);
+
+  if (gtk_widget_has_visible_focus (widget))
+    {
+      cairo_set_line_width (cr, 2);
+      if (swatch->priv->has_color && INTENSITY (swatch->priv->color.red, swatch->priv->color.green, swatch->priv->color.blue) < 0.5)
+        cairo_set_source_rgba (cr, 1., 1., 1., 0.4);
+      else
+        cairo_set_source_rgba (cr, 0., 0., 0., 0.4);
+      _gtk_rounded_box_shrink (&background.clip_box, 3, 3, 3, 3);
+      _gtk_rounded_box_path (&background.clip_box, cr);
+      cairo_stroke (cr);
     }
 
-  gtk_css_gadget_draw (swatch->priv->overlay_gadget, cr);
+  if (swatch->priv->icon)
+    {
+      icon_info = gtk_icon_theme_lookup_icon (theme, swatch->priv->icon, 16,
+                                              GTK_ICON_LOOKUP_GENERIC_FALLBACK
+                                              | GTK_ICON_LOOKUP_USE_BUILTIN);
+    }
+  else if ((state & GTK_STATE_FLAG_SELECTED) != 0)
+    {
+      GdkRGBA bg, border;
+      GtkBorder border_width;
+      GIcon *gicon;
 
-  return gtk_widget_has_visible_focus (widget);
+      gtk_style_context_add_class (context, "color-active-badge");
+      _gtk_theming_background_init_from_context (&background, context,
+                                                 (width - 2 * ACTIVE_BADGE_RADIUS) / 2, (height - 2 * ACTIVE_BADGE_RADIUS) / 2,
+                                                 2 * ACTIVE_BADGE_RADIUS, 2* ACTIVE_BADGE_RADIUS,
+                                                 GTK_JUNCTION_NONE);
+
+      if (_gtk_theming_background_has_background_image (&background))
+        {
+          _gtk_theming_background_render (&background, cr);
+        }
+      else
+        {
+          gtk_style_context_get_background_color (context, state, &bg);
+          gtk_style_context_get_border_color (context, state, &border);
+          gtk_style_context_get_border (context, state, &border_width);
+
+          cairo_new_sub_path (cr);
+          cairo_arc (cr, width / 2, height / 2, ACTIVE_BADGE_RADIUS, 0, 2 * G_PI);
+          cairo_close_path (cr);
+          gdk_cairo_set_source_rgba (cr, &bg);
+          cairo_fill_preserve (cr);
+
+          gdk_cairo_set_source_rgba (cr, &border);
+          cairo_set_line_width (cr, border_width.left);
+          cairo_stroke (cr);
+
+          gicon = g_themed_icon_new ("object-select-symbolic");
+          /* fallback for themes that don't have object-select-symbolic */
+          g_themed_icon_append_name (G_THEMED_ICON (gicon), "gtk-apply");
+
+          icon_info = gtk_icon_theme_lookup_by_gicon (theme, gicon, 16,
+                                                      GTK_ICON_LOOKUP_GENERIC_FALLBACK
+                                                      | GTK_ICON_LOOKUP_USE_BUILTIN);
+          g_object_unref (gicon);
+        }
+    }
+
+  if (icon_info != NULL)
+    {
+      GdkPixbuf *pixbuf;
+
+      pixbuf = gtk_icon_info_load_symbolic_for_context (icon_info, context,
+                                                        NULL, NULL);
+
+      if (pixbuf != NULL)
+        {
+          gtk_render_icon (context, cr, pixbuf,
+                           (width - gdk_pixbuf_get_width (pixbuf)) / 2,
+                           (height - gdk_pixbuf_get_height (pixbuf)) / 2);
+          g_object_unref (pixbuf);
+        }
+
+      gtk_icon_info_free (icon_info);
+    }
+
+  cairo_restore (cr);
+  gtk_style_context_restore (context);
+
+  return FALSE;
 }
 
 static void
@@ -246,7 +312,7 @@ swatch_drag_data_received (GtkWidget        *widget,
    */
   if (length != 8)
     {
-      g_warning ("Received invalid color data");
+      g_warning ("Received invalid color data\n");
       return;
     }
 
@@ -261,36 +327,19 @@ swatch_drag_data_received (GtkWidget        *widget,
 }
 
 static void
-gtk_color_swatch_measure (GtkCssGadget   *gadget,
-                          GtkOrientation  orientation,
-                          int             for_size,
-                          int            *minimum,
-                          int            *natural,
-                          int            *minimum_baseline,
-                          int            *natural_baseline,
-                          gpointer        unused)
+swatch_get_preferred_width (GtkWidget *widget,
+                            gint      *min,
+                            gint      *nat)
 {
-  GtkWidget *widget;
-  GtkColorSwatch *swatch;
-  gint w, h, min;
+  *min = *nat = 48;
+}
 
-  widget = gtk_css_gadget_get_owner (gadget);
-  swatch = GTK_COLOR_SWATCH (widget);
-
-  gtk_css_gadget_get_preferred_size (swatch->priv->overlay_gadget,
-                                     orientation,
-                                     -1,
-                                     minimum, natural,
-                                     NULL, NULL);
-
-  gtk_widget_get_size_request (widget, &w, &h);
-  if (orientation == GTK_ORIENTATION_HORIZONTAL)
-    min = w < 0 ? 48 : w;
-  else
-    min = h < 0 ? 32 : h;
-
-  *minimum = MAX (*minimum, min);
-  *natural = MAX (*natural, min);
+static void
+swatch_get_preferred_height (GtkWidget *widget,
+                             gint      *min,
+                             gint      *nat)
+{
+  *min = *nat = 32;
 }
 
 static gboolean
@@ -305,7 +354,7 @@ swatch_key_press (GtkWidget   *widget,
       event->keyval == GDK_KEY_KP_Enter ||
       event->keyval == GDK_KEY_KP_Space)
     {
-      if (swatch->priv->has_color &&
+      if (swatch->priv->has_color && 
           swatch->priv->selectable &&
           (gtk_widget_get_state_flags (widget) & GTK_STATE_FLAG_SELECTED) == 0)
         gtk_widget_set_state_flags (widget, GTK_STATE_FLAG_SELECTED, FALSE);
@@ -324,8 +373,8 @@ static gboolean
 swatch_enter_notify (GtkWidget        *widget,
                      GdkEventCrossing *event)
 {
-  gtk_widget_set_state_flags (widget, GTK_STATE_FLAG_PRELIGHT, FALSE);
-
+  GtkColorSwatch *swatch = GTK_COLOR_SWATCH (widget);
+  swatch->priv->contains_pointer = TRUE;
   return FALSE;
 }
 
@@ -333,8 +382,8 @@ static gboolean
 swatch_leave_notify (GtkWidget        *widget,
                      GdkEventCrossing *event)
 {
-  gtk_widget_unset_state_flags (widget, GTK_STATE_FLAG_PRELIGHT);
-
+  GtkColorSwatch *swatch = GTK_COLOR_SWATCH (widget);
+  swatch->priv->contains_pointer = FALSE;
   return FALSE;
 }
 
@@ -345,84 +394,119 @@ emit_customize (GtkColorSwatch *swatch)
 }
 
 static void
-do_popup (GtkColorSwatch *swatch)
+popup_position_func (GtkMenu   *menu,
+                     gint      *x,
+                     gint      *y,
+                     gboolean  *push_in,
+                     gpointer   user_data)
 {
-  if (swatch->priv->popover == NULL)
-    {
-      GtkWidget *box;
-      GtkWidget *item;
+  GtkWidget *widget;
+  GtkRequisition req;
+  gint root_x, root_y;
+  GdkScreen *screen;
+  GdkWindow *window;
+  GdkRectangle monitor;
+  gint monitor_num;
 
-      swatch->priv->popover = gtk_popover_new (GTK_WIDGET (swatch));
-      box = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
-      gtk_container_add (GTK_CONTAINER (swatch->priv->popover), box);
-      g_object_set (box, "margin", 10, NULL);
-      item = g_object_new (GTK_TYPE_MODEL_BUTTON,
-                           "text", _("C_ustomize"),
-                           NULL);
-      g_signal_connect_swapped (item, "clicked",
-                                G_CALLBACK (emit_customize), swatch);
-      gtk_container_add (GTK_CONTAINER (box), item);
-      gtk_widget_show_all (box);
-    }
+  widget = GTK_WIDGET (user_data);
+  g_return_if_fail (gtk_widget_get_realized (widget));
+  window = gtk_widget_get_window (widget);
 
-  gtk_popover_popup (GTK_POPOVER (swatch->priv->popover));
+  screen = gtk_widget_get_screen (widget);
+  monitor_num = gdk_screen_get_monitor_at_window (screen, window);
+  if (monitor_num < 0)
+    monitor_num = 0;
+  gtk_menu_set_monitor (menu, monitor_num);
+
+  gdk_window_get_origin (window, &root_x, &root_y);
+  gtk_widget_get_preferred_size (GTK_WIDGET (menu), &req, NULL);
+
+  /* Put corner of menu centered on swatch */
+  *x = root_x + gtk_widget_get_allocated_width (widget) / 2;
+  *y = root_y + gtk_widget_get_allocated_height (widget) / 2;
+
+  /* Ensure sanity */
+  gdk_screen_get_monitor_workarea (screen, monitor_num, &monitor);
+  *x = CLAMP (*x, monitor.x, MAX (monitor.x, monitor.width - req.width));
+  *y = CLAMP (*y, monitor.y, MAX (monitor.y, monitor.height - req.height));
+}
+
+static void
+do_popup (GtkWidget      *swatch,
+          GdkEventButton *event)
+{
+  GtkWidget *menu;
+  GtkWidget *item;
+
+  menu = gtk_menu_new ();
+  item = gtk_menu_item_new_with_mnemonic (_("_Customize"));
+  gtk_menu_attach_to_widget (GTK_MENU (menu), swatch, NULL);
+
+  g_signal_connect_swapped (item, "activate",
+                            G_CALLBACK (emit_customize), swatch);
+
+  gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+
+  gtk_widget_show_all (item);
+
+  if (event)
+    gtk_menu_popup (GTK_MENU (menu), NULL, NULL,
+                    NULL, NULL, event->button, event->time);
+  else
+    gtk_menu_popup (GTK_MENU (menu), NULL, NULL,
+                    popup_position_func, swatch,
+                    0, gtk_get_current_event_time ());
 }
 
 static gboolean
-swatch_primary_action (GtkColorSwatch *swatch)
+swatch_button_press (GtkWidget      *widget,
+                     GdkEventButton *event)
 {
-  GtkWidget *widget = (GtkWidget *)swatch;
-  GtkStateFlags flags;
+  GtkColorSwatch *swatch = GTK_COLOR_SWATCH (widget);
 
-  flags = gtk_widget_get_state_flags (widget);
-  if (!swatch->priv->has_color)
+  gtk_widget_grab_focus (widget);
+
+  if (gdk_event_triggers_context_menu ((GdkEvent *) event) &&
+      swatch->priv->has_color)
     {
-      g_signal_emit (swatch, signals[ACTIVATE], 0);
+      do_popup (widget, event);
       return TRUE;
     }
-  else if (swatch->priv->selectable &&
-           (flags & GTK_STATE_FLAG_SELECTED) == 0)
+  else if (event->type == GDK_2BUTTON_PRESS &&
+           event->button == GDK_BUTTON_PRIMARY)
     {
-      gtk_widget_set_state_flags (widget, GTK_STATE_FLAG_SELECTED, FALSE);
+      g_signal_emit (swatch, signals[ACTIVATE], 0);
       return TRUE;
     }
 
   return FALSE;
 }
 
-static void
-hold_action (GtkGestureLongPress *gesture,
-             gdouble              x,
-             gdouble              y,
-             GtkColorSwatch      *swatch)
+static gboolean
+swatch_button_release (GtkWidget      *widget,
+                       GdkEventButton *event)
 {
-  do_popup (swatch);
-  gtk_gesture_set_state (GTK_GESTURE (gesture), GTK_EVENT_SEQUENCE_CLAIMED);
-}
+  GtkColorSwatch *swatch = GTK_COLOR_SWATCH (widget);
+  GtkStateFlags flags;
 
-static void
-tap_action (GtkGestureMultiPress *gesture,
-            gint                  n_press,
-            gdouble               x,
-            gdouble               y,
-            GtkColorSwatch       *swatch)
-{
-  guint button;
-
-  button = gtk_gesture_single_get_current_button (GTK_GESTURE_SINGLE (gesture));
-
-  if (button == GDK_BUTTON_PRIMARY)
+  if (event->button == GDK_BUTTON_PRIMARY &&
+      swatch->priv->contains_pointer)
     {
-      if (n_press == 1)
-        swatch_primary_action (swatch);
-      else if (n_press > 1)
-        g_signal_emit (swatch, signals[ACTIVATE], 0);
-    }
-  else if (button == GDK_BUTTON_SECONDARY)
-    {
-      if (swatch->priv->has_color && swatch->priv->has_menu)
-        do_popup (swatch);
+      flags = gtk_widget_get_state_flags (widget);
+      if (!swatch->priv->has_color)
+        {
+          g_signal_emit (swatch, signals[ACTIVATE], 0);
+          return TRUE;
+        }
+      else if (swatch->priv->selectable &&
+               (flags & GTK_STATE_FLAG_SELECTED) == 0)
+        {
+          gtk_widget_set_state_flags (widget, GTK_STATE_FLAG_SELECTED, FALSE);
+          return TRUE;
+        }
     }
+
+  return FALSE;
 }
 
 static void
@@ -466,11 +550,10 @@ swatch_realize (GtkWidget *widget)
   attributes.height = allocation.height;
   attributes.wclass = GDK_INPUT_ONLY;
   attributes.event_mask = gtk_widget_get_events (widget);
-  attributes.event_mask |= GDK_BUTTON_PRESS_MASK
-                           | GDK_BUTTON_RELEASE_MASK
-                           | GDK_ENTER_NOTIFY_MASK
-                           | GDK_LEAVE_NOTIFY_MASK
-                           | GDK_TOUCH_MASK;
+  attributes.event_mask |= (GDK_BUTTON_PRESS_MASK |
+			    GDK_BUTTON_RELEASE_MASK |
+			    GDK_ENTER_NOTIFY_MASK |
+			    GDK_LEAVE_NOTIFY_MASK);
 
   attributes_mask = GDK_WA_X | GDK_WA_Y;
 
@@ -478,8 +561,10 @@ swatch_realize (GtkWidget *widget)
   gtk_widget_set_window (widget, window);
   g_object_ref (window);
 
-  swatch->priv->event_window = gdk_window_new (window, &attributes, attributes_mask);
-  gtk_widget_register_window (widget, swatch->priv->event_window);
+  swatch->priv->event_window = 
+    gdk_window_new (window,
+                    &attributes, attributes_mask);
+  gdk_window_set_user_data (swatch->priv->event_window, widget);
 }
 
 static void
@@ -489,7 +574,7 @@ swatch_unrealize (GtkWidget *widget)
 
   if (swatch->priv->event_window)
     {
-      gtk_widget_unregister_window (widget, swatch->priv->event_window);
+      gdk_window_set_user_data (swatch->priv->event_window, NULL);
       gdk_window_destroy (swatch->priv->event_window);
       swatch->priv->event_window = NULL;
     }
@@ -498,98 +583,28 @@ swatch_unrealize (GtkWidget *widget)
 }
 
 static void
-swatch_size_allocate (GtkWidget     *widget,
+swatch_size_allocate (GtkWidget *widget,
                       GtkAllocation *allocation)
 {
   GtkColorSwatch *swatch = GTK_COLOR_SWATCH (widget);
-  GtkAllocation clip, clip2;
 
   gtk_widget_set_allocation (widget, allocation);
 
-  gtk_css_gadget_allocate (swatch->priv->gadget,
-                           allocation,
-                           gtk_widget_get_allocated_baseline (widget),
-                           &clip);
-  gtk_css_gadget_allocate (swatch->priv->overlay_gadget,
-                           allocation,
-                           gtk_widget_get_allocated_baseline (widget),
-                           &clip2);
-
-  gdk_rectangle_union (&clip, &clip2, &clip);
-
-  gtk_widget_set_clip (widget, &clip);
-
   if (gtk_widget_get_realized (widget))
-    {
-      GtkAllocation border_allocation;
-      gtk_css_gadget_get_border_allocation(swatch->priv->gadget, &border_allocation, NULL);
-      gdk_window_move_resize (swatch->priv->event_window,
-                              border_allocation.x,
-                              border_allocation.y,
-                              border_allocation.width,
-                              border_allocation.height);
-    }
-
-}
-
-static void
-swatch_get_preferred_width (GtkWidget *widget,
-                            gint      *minimum,
-                            gint      *natural)
-{
-  gtk_css_gadget_get_preferred_size (GTK_COLOR_SWATCH (widget)->priv->gadget,
-                                     GTK_ORIENTATION_HORIZONTAL,
-                                     -1,
-                                     minimum, natural,
-                                     NULL, NULL);
-}
-
-static void
-swatch_get_preferred_height (GtkWidget *widget,
-                             gint      *minimum,
-                             gint      *natural)
-{
-  gtk_css_gadget_get_preferred_size (GTK_COLOR_SWATCH (widget)->priv->gadget,
-                                     GTK_ORIENTATION_VERTICAL,
-                                     -1,
-                                     minimum, natural,
-                                     NULL, NULL);
+    gdk_window_move_resize (swatch->priv->event_window,
+                            allocation->x,
+                            allocation->y,
+                            allocation->width,
+                            allocation->height);
 }
 
 static gboolean
-swatch_popup_menu (GtkWidget *widget)
+swatch_popup_menu (GtkWidget *swatch)
 {
-  do_popup (GTK_COLOR_SWATCH (widget));
+  do_popup (swatch, NULL);
   return TRUE;
 }
 
-static void
-update_icon (GtkColorSwatch *swatch)
-{
-  GtkIconHelper *icon_helper = GTK_ICON_HELPER (swatch->priv->overlay_gadget);
-
-  if (swatch->priv->icon)
-    _gtk_icon_helper_set_icon_name (icon_helper, swatch->priv->icon, GTK_ICON_SIZE_BUTTON);
-  else if (gtk_widget_get_state_flags (GTK_WIDGET (swatch)) & GTK_STATE_FLAG_SELECTED)
-    _gtk_icon_helper_set_icon_name (icon_helper, "object-select-symbolic", GTK_ICON_SIZE_BUTTON);
-  else
-    _gtk_icon_helper_clear (icon_helper);
-}
-
-static void
-swatch_state_flags_changed (GtkWidget     *widget,
-                            GtkStateFlags  previous_state)
-{
-  GtkColorSwatch *swatch = GTK_COLOR_SWATCH (widget);
-
-  gtk_css_gadget_set_state (swatch->priv->gadget, gtk_widget_get_state_flags (widget));
-  gtk_css_gadget_set_state (swatch->priv->overlay_gadget, gtk_widget_get_state_flags (widget));
-
-  update_icon (swatch);
-
-  GTK_WIDGET_CLASS (gtk_color_swatch_parent_class)->state_flags_changed (widget, previous_state);
-}
-
 /* GObject implementation {{{1 */
 
 static void
@@ -610,9 +625,6 @@ swatch_get_property (GObject    *object,
     case PROP_SELECTABLE:
       g_value_set_boolean (value, gtk_color_swatch_get_selectable (swatch));
       break;
-    case PROP_HAS_MENU:
-      g_value_set_boolean (value, swatch->priv->has_menu);
-      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -635,9 +647,6 @@ swatch_set_property (GObject      *object,
     case PROP_SELECTABLE:
       gtk_color_swatch_set_selectable (swatch, g_value_get_boolean (value));
       break;
-    case PROP_HAS_MENU:
-      swatch->priv->has_menu = g_value_get_boolean (value);
-      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -650,29 +659,10 @@ swatch_finalize (GObject *object)
   GtkColorSwatch *swatch = GTK_COLOR_SWATCH (object);
 
   g_free (swatch->priv->icon);
-  g_clear_object (&swatch->priv->gadget);
-  g_clear_object (&swatch->priv->overlay_gadget);
 
   G_OBJECT_CLASS (gtk_color_swatch_parent_class)->finalize (object);
 }
 
-static void
-swatch_dispose (GObject *object)
-{
-  GtkColorSwatch *swatch = GTK_COLOR_SWATCH (object);
-
-  if (swatch->priv->popover)
-    {
-      gtk_widget_destroy (swatch->priv->popover);
-      swatch->priv->popover = NULL;
-    }
-
-  g_clear_object (&swatch->priv->long_press_gesture);
-  g_clear_object (&swatch->priv->multipress_gesture);
-
-  G_OBJECT_CLASS (gtk_color_swatch_parent_class)->dispose (object);
-}
-
 static void
 gtk_color_swatch_class_init (GtkColorSwatchClass *class)
 {
@@ -682,7 +672,6 @@ gtk_color_swatch_class_init (GtkColorSwatchClass *class)
   object_class->get_property = swatch_get_property;
   object_class->set_property = swatch_set_property;
   object_class->finalize = swatch_finalize;
-  object_class->dispose = swatch_dispose;
 
   widget_class->get_preferred_width = swatch_get_preferred_width;
   widget_class->get_preferred_height = swatch_get_preferred_height;
@@ -692,6 +681,8 @@ gtk_color_swatch_class_init (GtkColorSwatchClass *class)
   widget_class->drag_data_received = swatch_drag_data_received;
   widget_class->key_press_event = swatch_key_press;
   widget_class->popup_menu = swatch_popup_menu;
+  widget_class->button_press_event = swatch_button_press;
+  widget_class->button_release_event = swatch_button_release;
   widget_class->enter_notify_event = swatch_enter_notify;
   widget_class->leave_notify_event = swatch_leave_notify;
   widget_class->realize = swatch_realize;
@@ -699,17 +690,16 @@ gtk_color_swatch_class_init (GtkColorSwatchClass *class)
   widget_class->map = swatch_map;
   widget_class->unmap = swatch_unmap;
   widget_class->size_allocate = swatch_size_allocate;
-  widget_class->state_flags_changed = swatch_state_flags_changed;
 
   signals[ACTIVATE] =
-    g_signal_new (I_("activate"),
+    g_signal_new ("activate",
                   GTK_TYPE_COLOR_SWATCH,
                   G_SIGNAL_RUN_FIRST,
                   G_STRUCT_OFFSET (GtkColorSwatchClass, activate),
                   NULL, NULL, NULL, G_TYPE_NONE, 0);
 
   signals[CUSTOMIZE] =
-    g_signal_new (I_("customize"),
+    g_signal_new ("customize",
                   GTK_TYPE_COLOR_SWATCH,
                   G_SIGNAL_RUN_FIRST,
                   G_STRUCT_OFFSET (GtkColorSwatchClass, customize),
@@ -721,52 +711,10 @@ gtk_color_swatch_class_init (GtkColorSwatchClass *class)
   g_object_class_install_property (object_class, PROP_SELECTABLE,
       g_param_spec_boolean ("selectable", P_("Selectable"), P_("Whether the swatch is selectable"),
                             TRUE, GTK_PARAM_READWRITE));
-  g_object_class_install_property (object_class, PROP_HAS_MENU,
-      g_param_spec_boolean ("has-menu", P_("Has Menu"), P_("Whether the swatch should offer customization"),
-                            TRUE, GTK_PARAM_READWRITE));
+
+  g_type_class_add_private (object_class, sizeof (GtkColorSwatchPrivate));
 
   gtk_widget_class_set_accessible_type (widget_class, GTK_TYPE_COLOR_SWATCH_ACCESSIBLE);
-  gtk_widget_class_set_css_name (widget_class, "colorswatch");
-}
-
-static void
-gtk_color_swatch_init (GtkColorSwatch *swatch)
-{
-  GtkCssNode *widget_node;
-
-  swatch->priv = gtk_color_swatch_get_instance_private (swatch);
-  swatch->priv->use_alpha = TRUE;
-  swatch->priv->selectable = TRUE;
-  swatch->priv->has_menu = TRUE;
-
-  gtk_widget_set_can_focus (GTK_WIDGET (swatch), TRUE);
-  gtk_widget_set_has_window (GTK_WIDGET (swatch), FALSE);
-
-  swatch->priv->long_press_gesture = gtk_gesture_long_press_new (GTK_WIDGET (swatch));
-  gtk_gesture_single_set_touch_only (GTK_GESTURE_SINGLE (swatch->priv->long_press_gesture),
-                                     TRUE);
-  g_signal_connect (swatch->priv->long_press_gesture, "pressed",
-                    G_CALLBACK (hold_action), swatch);
-
-  swatch->priv->multipress_gesture = gtk_gesture_multi_press_new (GTK_WIDGET (swatch));
-  gtk_gesture_single_set_button (GTK_GESTURE_SINGLE (swatch->priv->multipress_gesture), 0);
-  g_signal_connect (swatch->priv->multipress_gesture, "pressed",
-                    G_CALLBACK (tap_action), swatch);
-
-  widget_node = gtk_widget_get_css_node (GTK_WIDGET (swatch));
-  swatch->priv->gadget = gtk_css_custom_gadget_new_for_node (widget_node,
-                                                             GTK_WIDGET (swatch),
-                                                             gtk_color_swatch_measure,
-                                                             NULL,
-                                                             gtk_color_swatch_render,
-                                                             NULL,
-                                                             NULL);
-  gtk_css_gadget_add_class (swatch->priv->gadget, "activatable");
-
-  swatch->priv->overlay_gadget = gtk_icon_helper_new_named ("overlay", GTK_WIDGET (swatch));
-  _gtk_icon_helper_set_force_scale_pixbuf (GTK_ICON_HELPER (swatch->priv->overlay_gadget), TRUE);
-  gtk_css_node_set_parent (gtk_css_gadget_get_node (swatch->priv->overlay_gadget), widget_node);
-
 }
 
 /* Public API {{{1 */
@@ -796,20 +744,19 @@ gtk_color_swatch_set_rgba (GtkColorSwatch *swatch,
                            dnd_targets, G_N_ELEMENTS (dnd_targets),
                            GDK_ACTION_COPY | GDK_ACTION_MOVE);
     }
+  else
+    {
+      gtk_style_context_remove_class (context, "color-light");
+      gtk_style_context_remove_class (context, "color-dark");
+    }
 
   swatch->priv->has_color = TRUE;
   swatch->priv->color = *color;
 
   if (INTENSITY (swatch->priv->color.red, swatch->priv->color.green, swatch->priv->color.blue) > 0.5)
-    {
-      gtk_style_context_add_class (context, "light");
-      gtk_style_context_remove_class (context, "dark");
-    }
+    gtk_style_context_add_class (context, "color-light");
   else
-    {
-      gtk_style_context_add_class (context, "dark");
-      gtk_style_context_remove_class (context, "light");
-    }
+    gtk_style_context_add_class (context, "color-dark");
 
   gtk_widget_queue_draw (GTK_WIDGET (swatch));
   g_object_notify (G_OBJECT (swatch), "rgba");
@@ -842,7 +789,6 @@ gtk_color_swatch_set_icon (GtkColorSwatch *swatch,
                            const gchar    *icon)
 {
   swatch->priv->icon = g_strdup (icon);
-  update_icon (swatch);
   gtk_widget_queue_draw (GTK_WIDGET (swatch));
 }
 
