@@ -12,7 +12,8 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library. If not, see <http://www.gnu.org/licenses/>.
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  *
  * Authors: Benjamin Otte <otte@gnome.org>
  */
@@ -34,26 +35,29 @@ G_BEGIN_DECLS
 typedef struct _GtkCssStyleProperty           GtkCssStyleProperty;
 typedef struct _GtkCssStylePropertyClass      GtkCssStylePropertyClass;
 
-typedef GtkCssValue *    (* GtkCssStylePropertyParseFunc)  (GtkCssStyleProperty    *property,
-                                                            GtkCssParser           *parser);
-typedef void             (* GtkCssStylePropertyQueryFunc)  (GtkCssStyleProperty    *property,
-                                                            const GtkCssValue      *cssvalue,
-                                                            GValue                 *value);
-typedef GtkCssValue *    (* GtkCssStylePropertyAssignFunc) (GtkCssStyleProperty    *property,
-                                                            const GValue           *value);
+typedef gboolean         (* GtkCssStylePropertyParseFunc)  (GtkCssStyleProperty    *property,
+                                                            GValue                 *value,
+                                                            GtkCssParser           *parser,
+                                                            GFile                  *base);
+typedef void             (* GtkCssStylePropertyPrintFunc)  (GtkCssStyleProperty    *property,
+                                                            const GValue           *value,
+                                                            GString                *string);
+typedef void             (* GtkCssStylePropertyComputeFunc)(GtkCssStyleProperty    *property,
+                                                            GValue                 *computed,
+                                                            GtkStyleContext        *context,
+                                                            const GValue           *specified);
 struct _GtkCssStyleProperty
 {
   GtkStyleProperty parent;
 
-  GtkCssValue *initial_value;
+  GType computed_type;
+  GValue initial_value;
   guint id;
-  GtkCssAffects affects;
   guint inherit :1;
-  guint animated :1;
 
   GtkCssStylePropertyParseFunc parse_value;
-  GtkCssStylePropertyQueryFunc query_value;
-  GtkCssStylePropertyAssignFunc assign_value;
+  GtkCssStylePropertyPrintFunc print_value;
+  GtkCssStylePropertyComputeFunc compute_value;
 };
 
 struct _GtkCssStylePropertyClass
@@ -71,20 +75,23 @@ guint                   _gtk_css_style_property_get_n_properties(void);
 GtkCssStyleProperty *   _gtk_css_style_property_lookup_by_id    (guint                   id);
 
 gboolean                _gtk_css_style_property_is_inherit      (GtkCssStyleProperty    *property);
-gboolean                _gtk_css_style_property_is_animated     (GtkCssStyleProperty    *property);
-GtkCssAffects           _gtk_css_style_property_get_affects     (GtkCssStyleProperty    *property);
-gboolean                _gtk_css_style_property_affects_size    (GtkCssStyleProperty    *property);
-gboolean                _gtk_css_style_property_affects_font    (GtkCssStyleProperty    *property);
 guint                   _gtk_css_style_property_get_id          (GtkCssStyleProperty    *property);
-GtkCssValue  *          _gtk_css_style_property_get_initial_value
+const GValue *          _gtk_css_style_property_get_initial_value
                                                                 (GtkCssStyleProperty    *property);
+GType                   _gtk_css_style_property_get_computed_type (GtkCssStyleProperty *property);
+GType                   _gtk_css_style_property_get_specified_type (GtkCssStyleProperty *property);
+gboolean                _gtk_css_style_property_is_specified_type (GtkCssStyleProperty  *property,
+                                                                 GType                   type);
+
+void                    _gtk_css_style_property_compute_value   (GtkCssStyleProperty    *property,
+                                                                 GValue                 *computed,
+                                                                 GtkStyleContext        *context,
+                                                                 const GValue           *specified);
 
 void                    _gtk_css_style_property_print_value     (GtkCssStyleProperty    *property,
-                                                                 GtkCssValue            *value,
+                                                                 const GValue           *value,
                                                                  GString                *string);
-
-GtkBitmask *            _gtk_css_style_property_get_mask_affecting
-                                                                (GtkCssAffects           affects);
+                                                                 
 
 G_END_DECLS
 
