@@ -80,16 +80,12 @@ void     _gtk_entry_completion_popup        (GtkEntryCompletion *completion,
 void     _gtk_entry_completion_popdown      (GtkEntryCompletion *completion);
 
 void      _gtk_entry_get_borders            (GtkEntry  *entry,
-					     gint      *xborder,
-					     gint      *yborder);
-void     _gtk_entry_effective_inner_border (GtkEntry  *entry,
-					    GtkBorder *border);
+                                             GtkBorder *borders);
 void     _gtk_entry_reset_im_context       (GtkEntry  *entry);
 GtkIMContext* _gtk_entry_get_im_context    (GtkEntry  *entry);
 void     _gtk_entry_set_is_cell_renderer   (GtkEntry  *entry,
                                             gboolean   is_cell_renderer);
 
-
 G_END_DECLS
 
 #endif /* __GTK_ENTRY_PRIVATE_H__ */
