@@ -12,7 +12,9 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library. If not, see <http://www.gnu.org/licenses/>.
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
 /*
@@ -22,14 +24,14 @@
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/.
  */
 
-#ifndef __GTK_MISC_H__
-#define __GTK_MISC_H__
-
-
 #if !defined (__GTK_H_INSIDE__) && !defined (GTK_COMPILATION)
 #error "Only <gtk/gtk.h> can be included directly."
 #endif
 
+#ifndef __GTK_MISC_H__
+#define __GTK_MISC_H__
+
+
 #include <gtk/gtkwidget.h>
 
 
@@ -66,21 +68,17 @@ struct _GtkMiscClass
   void (*_gtk_reserved4) (void);
 };
 
-GDK_AVAILABLE_IN_ALL
+
 GType   gtk_misc_get_type      (void) G_GNUC_CONST;
-GDK_AVAILABLE_IN_ALL
 void	gtk_misc_set_alignment (GtkMisc *misc,
 				gfloat	 xalign,
 				gfloat	 yalign);
-GDK_AVAILABLE_IN_ALL
 void    gtk_misc_get_alignment (GtkMisc *misc,
 				gfloat  *xalign,
 				gfloat  *yalign);
-GDK_AVAILABLE_IN_ALL
 void	gtk_misc_set_padding   (GtkMisc *misc,
 				gint	 xpad,
 				gint	 ypad);
-GDK_AVAILABLE_IN_ALL
 void    gtk_misc_get_padding   (GtkMisc *misc,
 				gint    *xpad,
 				gint    *ypad);
