@@ -13,7 +13,9 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library. If not, see <http://www.gnu.org/licenses/>.
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
 #include "config.h"
@@ -816,9 +818,8 @@ gtk_print_settings_set_use_color (GtkPrintSettings *settings,
 gboolean
 gtk_print_settings_get_collate (GtkPrintSettings *settings)
 {
-  return gtk_print_settings_get_bool_with_default (settings,
-                                                   GTK_PRINT_SETTINGS_COLLATE,
-                                                   TRUE);
+  return gtk_print_settings_get_bool (settings, 
+				      GTK_PRINT_SETTINGS_COLLATE);
 }
 
 /**
