@@ -15,7 +15,9 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library. If not, see <http://www.gnu.org/licenses/>.
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
 #include "config.h"
@@ -35,13 +37,13 @@
  * that is, a button that is part of a group of toggle buttons where only
  * one button can be active at a time.
  *
- * Use gtk_radio_tool_button_new() to create a new GtkRadioToolButton. Use
- * gtk_radio_tool_button_new_from_widget() to create a new GtkRadioToolButton
- * that is part of the same group as an existing GtkRadioToolButton.
- *
- * # CSS nodes
- *
- * GtkRadioToolButton has a single CSS node with name toolbutton.
+ * Use gtk_radio_tool_button_new() to create a new
+ * #GtkRadioToolButton. Use gtk_radio_tool_button_new_from_widget() to
+ * create a new #GtkRadioToolButton that is part of the same group as an
+ * existing #GtkRadioToolButton. Use
+ * gtk_radio_tool_button_new_from_stock() or
+ * gtk_radio_tool_button_new_with_stock_from_widget() create a new
+ * #GtkRadioToolButton containing a stock item.
  */
 
 
@@ -127,12 +129,12 @@ gtk_radio_tool_button_set_property (GObject         *object,
 
 /**
  * gtk_radio_tool_button_new:
- * @group: (allow-none) (element-type GtkRadioButton): An
+ * @group: (allow-none) (transfer none) (element-type GtkRadioButton): An
  *   existing radio button group, or %NULL if you are creating a new group
  * 
  * Creates a new #GtkRadioToolButton, adding it to @group.
  * 
- * Returns: The new #GtkRadioToolButton
+ * Return value: The new #GtkRadioToolButton
  * 
  * Since: 2.4
  **/
@@ -151,19 +153,17 @@ gtk_radio_tool_button_new (GSList *group)
 
 /**
  * gtk_radio_tool_button_new_from_stock:
- * @group: (allow-none) (element-type GtkRadioButton): an existing radio button
- *   group, or %NULL if you are creating a new group
+ * @group: (allow-none): an existing radio button group, or %NULL if you are
+ *   creating a new group
  * @stock_id: the name of a stock item
  * 
  * Creates a new #GtkRadioToolButton, adding it to @group. 
  * The new #GtkRadioToolButton will contain an icon and label from the
  * stock item indicated by @stock_id.
  * 
- * Returns: The new #GtkRadioToolButton
+ * Return value: The new #GtkRadioToolItem
  * 
  * Since: 2.4
- *
- * Deprecated: 3.10: Use gtk_radio_tool_button_new() instead.
  **/
 GtkToolItem *
 gtk_radio_tool_button_new_from_stock (GSList      *group,
@@ -189,7 +189,7 @@ gtk_radio_tool_button_new_from_stock (GSList      *group,
  *
  * Creates a new #GtkRadioToolButton adding it to the same group as @gruup
  *
- * Returns: (transfer none): The new #GtkRadioToolButton
+ * Return value: (transfer none): The new #GtkRadioToolButton
  *
  * Since: 2.4
  **/
@@ -215,29 +215,22 @@ gtk_radio_tool_button_new_from_widget (GtkRadioToolButton *group)
  * The new #GtkRadioToolButton will contain an icon and label from the
  * stock item indicated by @stock_id.
  *
- * Returns: (transfer none): A new #GtkRadioToolButton
+ * Return value: (transfer none): A new #GtkRadioToolButton
  *
  * Since: 2.4
- *
- * Deprecated: 3.10: gtk_radio_tool_button_new_from_widget
  **/
 GtkToolItem *
 gtk_radio_tool_button_new_with_stock_from_widget (GtkRadioToolButton *group,
 						  const gchar        *stock_id)
 {
   GSList *list = NULL;
-  GtkToolItem *item;
-
+  
   g_return_val_if_fail (group == NULL || GTK_IS_RADIO_TOOL_BUTTON (group), NULL);
 
   if (group != NULL)
     list = gtk_radio_tool_button_get_group (group);
-
-  G_GNUC_BEGIN_IGNORE_DEPRECATIONS;
-  item = gtk_radio_tool_button_new_from_stock (list, stock_id);
-  G_GNUC_END_IGNORE_DEPRECATIONS;
-
-  return item;
+  
+  return gtk_radio_tool_button_new_from_stock (list, stock_id);
 }
 
 static GtkRadioButton *
@@ -252,7 +245,7 @@ get_radio_button (GtkRadioToolButton *button)
  *
  * Returns the radio button group @button belongs to.
  *
- * Returns: (transfer none) (element-type GtkRadioButton): The group @button belongs to.
+ * Return value: (transfer none) (element-type GtkRadioButton): The group @button belongs to.
  *
  * Since: 2.4
  */
@@ -267,7 +260,7 @@ gtk_radio_tool_button_get_group (GtkRadioToolButton *button)
 /**
  * gtk_radio_tool_button_set_group:
  * @button: a #GtkRadioToolButton
- * @group: (element-type GtkRadioButton) (allow-none): an existing radio button group, or %NULL
+ * @group: (transfer none) (element-type GtkRadioButton): an existing radio button group
  * 
  * Adds @button to @group, removing it from the group it belonged to before.
  * 
