@@ -182,9 +182,6 @@ struct _GtkThemingEngineClass
 
 GType gtk_theming_engine_get_type (void) G_GNUC_CONST;
 
-void _gtk_theming_engine_set_context (GtkThemingEngine *engine,
-                                      GtkStyleContext  *context);
-
 /* function implemented in gtkcsscustomproperty.c */
 void gtk_theming_engine_register_property (const gchar            *name_space,
                                            GtkStylePropertyParser  parse_func,
