@@ -15,7 +15,9 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library. If not, see <http://www.gnu.org/licenses/>.
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
 #include "config.h"
@@ -28,7 +30,7 @@
 #include "gtktoolshell.h"
 #include "gtkseparatormenuitem.h"
 #include "gtksizerequest.h"
-#include "deprecated/gtkactivatable.h"
+#include "gtkactivatable.h"
 #include "gtkintl.h"
 #include "gtkprivate.h"
 
@@ -153,12 +155,9 @@ static void gtk_tool_item_set_use_action_appearance  (GtkToolItem          *item
 
 static guint toolitem_signals[LAST_SIGNAL] = { 0 };
 
-G_GNUC_BEGIN_IGNORE_DEPRECATIONS;
 G_DEFINE_TYPE_WITH_CODE (GtkToolItem, gtk_tool_item, GTK_TYPE_BIN,
-                         G_ADD_PRIVATE (GtkToolItem)
 			 G_IMPLEMENT_INTERFACE (GTK_TYPE_ACTIVATABLE,
 						gtk_tool_item_activatable_interface_init))
-G_GNUC_END_IGNORE_DEPRECATIONS;
 
 static void
 gtk_tool_item_class_init (GtkToolItemClass *klass)
@@ -276,6 +275,8 @@ gtk_tool_item_class_init (GtkToolItemClass *klass)
 		  NULL, NULL,
 		  _gtk_marshal_VOID__VOID,
 		  G_TYPE_NONE, 0);
+
+  g_type_class_add_private (object_class, sizeof (GtkToolItemPrivate));
 }
 
 static void
@@ -283,11 +284,15 @@ gtk_tool_item_init (GtkToolItem *toolitem)
 {
   gtk_widget_set_can_focus (GTK_WIDGET (toolitem), FALSE);
 
-  toolitem->priv = gtk_tool_item_get_instance_private (toolitem);
+  toolitem->priv = G_TYPE_INSTANCE_GET_PRIVATE (toolitem,
+                                                GTK_TYPE_TOOL_ITEM,
+                                                GtkToolItemPrivate);
+
   toolitem->priv->visible_horizontal = TRUE;
   toolitem->priv->visible_vertical = TRUE;
   toolitem->priv->homogeneous = FALSE;
   toolitem->priv->expand = FALSE;
+
   toolitem->priv->use_action_appearance = TRUE;
 }
 
@@ -311,9 +316,7 @@ gtk_tool_item_dispose (GObject *object)
 
   if (item->priv->action)
     {
-      G_GNUC_BEGIN_IGNORE_DEPRECATIONS;
       gtk_activatable_do_set_related_action (GTK_ACTIVATABLE (item), NULL);      
-      G_GNUC_END_IGNORE_DEPRECATIONS;
       item->priv->action = NULL;
     }
   G_OBJECT_CLASS (gtk_tool_item_parent_class)->dispose (object);
@@ -399,9 +402,6 @@ gtk_tool_item_property_notify (GObject    *object,
   if (tool_item->priv->menu_item && strcmp (pspec->name, "sensitive") == 0)
     gtk_widget_set_sensitive (tool_item->priv->menu_item,
 			      gtk_widget_get_sensitive (GTK_WIDGET (tool_item)));
-
-  if (G_OBJECT_CLASS (gtk_tool_item_parent_class)->notify)
-    G_OBJECT_CLASS (gtk_tool_item_parent_class)->notify (object, pspec);
 }
 
 static void
@@ -431,7 +431,7 @@ create_drag_window (GtkToolItem *toolitem)
 
   toolitem->priv->drag_window = gdk_window_new (gtk_widget_get_parent_window (widget),
 					  &attributes, attributes_mask);
-  gtk_widget_register_window (widget, toolitem->priv->drag_window);
+  gdk_window_set_user_data (toolitem->priv->drag_window, toolitem);
 }
 
 static void
@@ -456,7 +456,7 @@ destroy_drag_window (GtkToolItem *toolitem)
 {
   if (toolitem->priv->drag_window)
     {
-      gtk_widget_unregister_window (GTK_WIDGET (toolitem), toolitem->priv->drag_window);
+      gdk_window_set_user_data (toolitem->priv->drag_window, NULL);
       gdk_window_destroy (toolitem->priv->drag_window);
       toolitem->priv->drag_window = NULL;
     }
@@ -558,9 +558,6 @@ _gtk_tool_item_create_menu_proxy (GtkToolItem *item)
 {
   GtkWidget *menu_item;
   gboolean visible_overflown;
-  gboolean ret = FALSE;
-
-  G_GNUC_BEGIN_IGNORE_DEPRECATIONS;
 
   if (item->priv->action)
     {
@@ -577,12 +574,10 @@ _gtk_tool_item_create_menu_proxy (GtkToolItem *item)
       else
 	gtk_tool_item_set_proxy_menu_item (item, "gtk-action-menu-item", NULL);
 
-      ret = TRUE;
+      return TRUE;
     }
 
-  G_GNUC_END_IGNORE_DEPRECATIONS;
-
-  return ret;
+  return FALSE;
 }
 
 static void
@@ -597,8 +592,6 @@ gtk_tool_item_update (GtkActivatable *activatable,
 		      GtkAction      *action,
 	     	      const gchar    *property_name)
 {
-  G_GNUC_BEGIN_IGNORE_DEPRECATIONS;
-
   if (strcmp (property_name, "visible") == 0)
     {
       if (gtk_action_is_visible (action))
@@ -620,8 +613,6 @@ gtk_tool_item_update (GtkActivatable *activatable,
   else if (strcmp (property_name, "is-important") == 0)
     gtk_tool_item_set_is_important (GTK_TOOL_ITEM (activatable),
 				    gtk_action_get_is_important (action));
-
-  G_GNUC_END_IGNORE_DEPRECATIONS;
 }
 
 static void
@@ -631,8 +622,6 @@ gtk_tool_item_sync_action_properties (GtkActivatable *activatable,
   if (!action)
     return;
 
-  G_GNUC_BEGIN_IGNORE_DEPRECATIONS;
-
   if (gtk_action_is_visible (action))
     gtk_widget_show (GTK_WIDGET (activatable));
   else
@@ -648,8 +637,6 @@ gtk_tool_item_sync_action_properties (GtkActivatable *activatable,
 				      gtk_action_get_visible_vertical (action));
   gtk_tool_item_set_is_important (GTK_TOOL_ITEM (activatable),
 				  gtk_action_get_is_important (action));
-
-  G_GNUC_END_IGNORE_DEPRECATIONS;
 }
 
 static void
@@ -659,9 +646,7 @@ gtk_tool_item_set_related_action (GtkToolItem *item,
   if (item->priv->action == action)
     return;
 
-  G_GNUC_BEGIN_IGNORE_DEPRECATIONS;
   gtk_activatable_do_set_related_action (GTK_ACTIVATABLE (item), action);
-  G_GNUC_END_IGNORE_DEPRECATIONS;
 
   item->priv->action = action;
 
@@ -679,9 +664,7 @@ gtk_tool_item_set_use_action_appearance (GtkToolItem *item,
     {
       item->priv->use_action_appearance = use_appearance;
 
-      G_GNUC_BEGIN_IGNORE_DEPRECATIONS;
       gtk_activatable_sync_action_properties (GTK_ACTIVATABLE (item), item->priv->action);
-      G_GNUC_END_IGNORE_DEPRECATIONS;
     }
 }
 
@@ -804,7 +787,10 @@ gtk_tool_item_get_orientation (GtkToolItem *tool_item)
  * <listitem> GTK_TOOLBAR_TEXT, meaning the tool item should only
  * show text</listitem>
  * <listitem> GTK_TOOLBAR_BOTH_HORIZ, meaning the tool item should show
- * both an icon and a label, arranged horizontally</listitem>
+ * both an icon and a label, arranged horizontally (however, note the 
+ * #GtkToolButton:has_text_horizontally property that makes tool buttons not
+ * show labels when the toolbar style is GTK_TOOLBAR_BOTH_HORIZ.
+ * </listitem>
  * </itemizedlist>
  * 
  * Return value: A #GtkToolbarStyle indicating the toolbar style used
@@ -830,7 +816,7 @@ gtk_tool_item_get_toolbar_style (GtkToolItem *tool_item)
  * gtk_tool_item_get_relief_style:
  * @tool_item: a #GtkToolItem 
  * 
- * Returns the relief style of @tool_item. See gtk_button_set_relief().
+ * Returns the relief style of @tool_item. See gtk_button_set_relief_style().
  * Custom subclasses of #GtkToolItem should call this function in the handler
  * of the #GtkToolItem::toolbar_reconfigured signal to find out the
  * relief style of buttons.
@@ -1425,7 +1411,7 @@ gtk_tool_item_toolbar_reconfigured (GtkToolItem *tool_item)
   /* The slightely inaccurate name "gtk_tool_item_toolbar_reconfigured" was
    * choosen over "gtk_tool_item_tool_shell_reconfigured", since the function
    * emits the "toolbar-reconfigured" signal, not "tool-shell-reconfigured".
-   * It's not possible to rename the signal, and emitting another name than
+   * Its not possible to rename the signal, and emitting another name than
    * indicated by the function name would be quite confusing. That's the
    * price of providing stable APIs.
    */
