@@ -46,8 +46,7 @@
 #include "gtkmenushellprivate.h"
 #include "gtkprivate.h"
 
-#undef GTK_DISABLE_DEPRECATED
-#include "gtktearoffmenuitem.h"
+#include "deprecated/gtktearoffmenuitem.h"
 
 /* GObjectClass */
 static GObject  *gtk_tree_menu_constructor                    (GType                  type,
