@@ -123,7 +123,6 @@
 *tp++ = gtk_app_chooser_dialog_get_type();
 *tp++ = gtk_app_chooser_get_type();
 *tp++ = gtk_app_chooser_widget_get_type();
-*tp++ = gtk_application_end_session_style_get_type();
 *tp++ = gtk_application_get_type();
 *tp++ = gtk_application_inhibit_flags_get_type();
 *tp++ = gtk_application_window_get_type();
@@ -169,6 +168,9 @@
 *tp++ = gtk_check_menu_item_get_type();
 *tp++ = gtk_clipboard_get_type();
 *tp++ = gtk_color_button_get_type();
+*tp++ = gtk_color_chooser_dialog_get_type();
+*tp++ = gtk_color_chooser_get_type();
+*tp++ = gtk_color_chooser_widget_get_type();
 *tp++ = gtk_color_selection_dialog_get_type();
 *tp++ = gtk_color_selection_get_type();
 *tp++ = gtk_combo_box_get_type();
