@@ -12,16 +12,17 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library. If not, see <http://www.gnu.org/licenses/>.
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
-#include "config.h"
-
 #include "gtkimcontextmultipress.h"
 #include <string.h>
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtkimmodule.h>
+#include <config.h>
 
 #define AUTOMATIC_COMPOSE_TIMEOUT 1 /* seconds */
 #define CONFIGURATION_FILENAME MULTIPRESS_CONFDIR G_DIR_SEPARATOR_S "im-multipress.conf"
@@ -170,14 +171,11 @@ clear_compose_buffer (GtkImContextMultipress *multipress_context)
   multipress_context->key_last_entered = 0;
   multipress_context->compose_count = 0;
 
+  multipress_context->tentative_match = NULL;
   cancel_automatic_timeout_commit (multipress_context);
 
-  if (multipress_context->tentative_match)
-    {
-      multipress_context->tentative_match = NULL;
-      g_signal_emit_by_name (multipress_context, "preedit-changed");
-      g_signal_emit_by_name (multipress_context, "preedit-end");
-    }
+  g_signal_emit_by_name (multipress_context, "preedit-changed");
+  g_signal_emit_by_name (multipress_context, "preedit-end");
 }
 
 /* Finish composing, provide the character, and clear our compose buffer.
@@ -201,7 +199,7 @@ on_timeout (gpointer data)
 {
   GtkImContextMultipress *multipress_context;
 
-  gdk_threads_enter ();
+  GDK_THREADS_ENTER ();
 
   multipress_context = GTK_IM_CONTEXT_MULTIPRESS (data);
 
@@ -211,7 +209,7 @@ on_timeout (gpointer data)
 
   multipress_context->timeout_id = 0;
 
-  gdk_threads_leave ();
+  GDK_THREADS_LEAVE ();
 
   return G_SOURCE_REMOVE; /* don't call me again */
 }
@@ -275,7 +273,6 @@ vfunc_filter_keypress (GtkIMContext *context, GdkEventKey *event)
           multipress_context->timeout_id =
             g_timeout_add_seconds (AUTOMATIC_COMPOSE_TIMEOUT,
                                    &on_timeout, multipress_context);
-          g_source_set_name_by_id (multipress_context->timeout_id, "[gtk+] on_timeout");
 
           return TRUE; /* key handled */
         }
