@@ -568,6 +568,12 @@ add_tests_for_files_in_directory (GFile *dir)
 int
 main (int argc, char **argv)
 {
+  /* I don't want to fight fuzzy scaling algorithms in GPUs,
+   * so unles you explicitly set it to something else, we
+   * will use Cairo's image surface.
+   */
+  g_setenv ("GDK_RENDERING", "image", FALSE);
+
   if (!parse_command_line (&argc, &argv))
     return 1;
 
