@@ -15,7 +15,9 @@
  * Library General Public License for more details.
  *
  * You should have received a copy of the GNU Library General Public
- * License along with this library. If not, see <http://www.gnu.org/licenses/>.
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
 #include <gtk/gtk.h>
@@ -37,8 +39,8 @@ static GtkWidget *test_widgets[TEST_WIDGET_LAST];
 static GtkWidget*
 create_image (void)
 {
-  return gtk_image_new_from_icon_name ("document-open",
-                                       GTK_ICON_SIZE_BUTTON);
+  return gtk_image_new_from_stock (GTK_STOCK_OPEN,
+                                   GTK_ICON_SIZE_BUTTON);
 }
 
 static GtkWidget*
